"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const archiver_1 = require("archiver");
const fs_1 = require("fs");
const path_1 = require("path");
const stream_1 = require("stream");
const util_1 = require("util");
const errors_1 = require("../errors");
const fileSystemHandler_1 = require("../utils/fileSystemHandler");
const convertTransaction_1 = require("./convertTransaction");
const transformers_1 = require("./transformers");
const constants_1 = require("../utils/constants");
const fast_xml_parser_1 = require("fast-xml-parser");
exports.pipeline = util_1.promisify(stream_1.pipeline);
class ComponentReader extends stream_1.Readable {
    constructor(components) {
        super({ objectMode: true });
        this.currentIndex = 0;
        this.components = components;
    }
    _read() {
        if (this.currentIndex < this.components.length) {
            const c = this.components[this.currentIndex];
            this.currentIndex += 1;
            this.push(c);
        }
        else {
            this.push(null);
        }
    }
}
exports.ComponentReader = ComponentReader;
class ComponentConverter extends stream_1.Transform {
    constructor(targetFormat, registry, transaction = new convertTransaction_1.ConvertTransaction()) {
        super({ objectMode: true });
        this.targetFormat = targetFormat;
        this.transaction = transaction;
        this.transformerFactory = new transformers_1.MetadataTransformerFactory(registry, this.transaction);
    }
    _transform(chunk, encoding, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let err;
            let result;
            try {
                const transformer = this.transformerFactory.getTransformer(chunk);
                switch (this.targetFormat) {
                    case 'metadata':
                        result = yield transformer.toMetadataFormat(chunk);
                        break;
                    case 'source':
                        result = yield transformer.toSourceFormat(chunk);
                        break;
                    default:
                        throw new errors_1.LibraryError('error_convert_invalid_format', this.targetFormat);
                }
            }
            catch (e) {
                err = e;
            }
            callback(err, result);
        });
    }
    /**
     * Called at the end when all components have passed through the pipeline. Finalizers
     * take care of any additional work to be done at this stage e.g. recomposing child components.
     */
    _flush(callback) {
        var e_1, _a;
        return __awaiter(this, void 0, void 0, function* () {
            let err;
            try {
                try {
                    for (var _b = __asyncValues(this.transaction.executeFinalizers()), _c; _c = yield _b.next(), !_c.done;) {
                        const finalizerResult = _c.value;
                        if (finalizerResult) {
                            if (Array.isArray(finalizerResult)) {
                                finalizerResult.forEach((result) => this.push(result));
                            }
                            else {
                                this.push(finalizerResult);
                            }
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
            catch (e) {
                err = e;
            }
            callback(err);
        });
    }
}
exports.ComponentConverter = ComponentConverter;
class ComponentWriter extends stream_1.Writable {
    constructor(rootDestination) {
        super({ objectMode: true });
        this.rootDestination = rootDestination;
    }
}
exports.ComponentWriter = ComponentWriter;
class StandardWriter extends ComponentWriter {
    constructor(rootDestination) {
        super(rootDestination);
    }
    _write(chunk, encoding, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let err;
            try {
                const infos = chunk.getExtraInfos ? yield chunk.getExtraInfos() : [];
                infos.push(...chunk.writeInfos);
                const writeTasks = infos.map((info) => {
                    const fullDest = path_1.join(this.rootDestination, info.relativeDestination);
                    fileSystemHandler_1.ensureFileExists(fullDest);
                    return exports.pipeline(info.source, fs_1.createWriteStream(fullDest));
                });
                // it is a reasonable expectation that when a conversion call exits, the files of
                // every component has been written to the destination. This await ensures the microtask
                // queue is empty when that call exits and overall less memory is consumed.
                yield Promise.all(writeTasks);
            }
            catch (e) {
                err = e;
            }
            callback(err);
        });
    }
}
exports.StandardWriter = StandardWriter;
class ZipWriter extends ComponentWriter {
    constructor(rootDestination) {
        super(rootDestination);
        // compression-/speed+ (0)<---(3)---------->(9) compression+/speed-
        // 3 appears to be a decent balance of compression and speed. It felt like
        // higher values = diminishing returns on compression and made conversion slower
        this.zip = archiver_1.create('zip', { zlib: { level: 3 } });
        this.buffers = [];
        exports.pipeline(this.zip, this.getOutputStream());
    }
    _write(chunk, encoding, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let err;
            try {
                const infos = chunk.getExtraInfos ? yield chunk.getExtraInfos() : [];
                infos.push(...chunk.writeInfos);
                for (const writeInfo of infos) {
                    this.addToZip(writeInfo.source, writeInfo.relativeDestination);
                }
            }
            catch (e) {
                err = e;
            }
            callback(err);
        });
    }
    _final(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let err;
            try {
                yield this.zip.finalize();
            }
            catch (e) {
                err = e;
            }
            callback(err);
        });
    }
    addToZip(contents, path) {
        this.zip.append(contents, { name: path });
    }
    getOutputStream() {
        if (this.rootDestination) {
            return fs_1.createWriteStream(this.rootDestination);
        }
        else {
            const bufferWritable = new stream_1.Writable();
            bufferWritable._write = (chunk, encoding, cb) => {
                this.buffers.push(chunk);
                cb();
            };
            return bufferWritable;
        }
    }
    get buffer() {
        return Buffer.concat(this.buffers);
    }
}
exports.ZipWriter = ZipWriter;
/**
 * Convenient wrapper to serialize a js object to XML content. Implemented as a stream
 * to be used as a valid source for ComponentWriters in the conversion pipeline,
 * even though it's not beneficial in the typical way a stream is.
 */
class JsToXml extends stream_1.Readable {
    constructor(xmlObject) {
        super();
        this.xmlObject = xmlObject;
    }
    _read() {
        const js2Xml = new fast_xml_parser_1.j2xParser({ format: true, indentBy: '    ', ignoreAttributes: false });
        const xmlContent = constants_1.XML_DECL.concat(js2Xml.parse(this.xmlObject));
        this.push(xmlContent);
        this.push(null);
    }
}
exports.JsToXml = JsToXml;
//# sourceMappingURL=streams.js.map