"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const testSetup_1 = require("@salesforce/core/lib/testSetup");
const chai_1 = require("chai");
const fs = require("fs");
const sinon_1 = require("sinon");
const deployStrategies_1 = require("../../../src/client/deployStrategies");
const i18n_1 = require("../../../src/i18n");
const types_1 = require("../../../src/client/types");
const metadata_registry_1 = require("../../../src/metadata-registry");
const $$ = testSetup_1.testSetup();
describe('Container Deploy Strategy', () => {
    let simpleMetaXMLString = '<?xml version="1.0" encoding="UTF-8"?>';
    simpleMetaXMLString += '<ApexClass xmlns="http://soap.sforce.com/2006/04/metadata">';
    simpleMetaXMLString += '    <apiVersion>32.0</apiVersion>';
    simpleMetaXMLString += '    <status>Active</status>';
    simpleMetaXMLString += '</ApexClass>';
    const successfulContainerResult = {
        success: true,
        id: '1dcxxx000000034',
        errors: [],
        name: 'VSCode_MDC_',
        message: '',
    };
    const apexClassCmp = new metadata_registry_1.SourceComponent({
        type: metadata_registry_1.registryData.types.apexclass,
        name: 'one',
        content: 'file/path/one.cls',
        xml: 'file/path/one.cls-meta.xml',
    });
    const apexTriggerCmp = new metadata_registry_1.SourceComponent({
        type: metadata_registry_1.registryData.types.apextrigger,
        name: 'one',
        content: 'file/path/one.trigger',
        xml: 'file/path/one.trigger-meta.xml',
    });
    const apexPageCmp = new metadata_registry_1.SourceComponent({
        type: metadata_registry_1.registryData.types.apexpage,
        name: 'one',
        content: 'file/path/one.page',
        xml: 'file/path/one.page-meta.xml',
    });
    const apexComponent = new metadata_registry_1.SourceComponent({
        type: metadata_registry_1.registryData.types.apexcomponent,
        name: 'one',
        content: 'file/path/one.component',
        xml: 'file/path/one.component-meta.xml',
    });
    const testData = new testSetup_1.MockTestOrgData();
    let mockConnection;
    let sandboxStub;
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
        $$.setConfigStubContents('AuthInfoConfig', {
            contents: yield testData.getConfig(),
        });
        mockConnection = yield core_1.Connection.create({
            authInfo: yield core_1.AuthInfo.create({
                username: testData.username,
            }),
        });
        const mockFS = sandboxStub.stub(fs, 'readFileSync');
        mockFS.withArgs('file/path/one.cls', 'utf8').returns('public with sharing class TestAPI {}');
        mockFS.withArgs('file/path/one.cls-meta.xml', 'utf8').returns(simpleMetaXMLString);
        mockFS.withArgs('file/path/one.component-meta.xml', 'utf8').returns(simpleMetaXMLString);
        mockFS.withArgs('file/path/one.page-meta.xml', 'utf8').returns(simpleMetaXMLString);
        mockFS.withArgs('file/path/one.trigger-meta.xml', 'utf8').returns(simpleMetaXMLString);
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('should create a metadata container', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create');
        mockToolingCreate.resolves({
            success: true,
            id: '1dcxxx000000034',
            errors: [],
        });
        const container = yield deployLibrary.createMetadataContainer();
        chai_1.expect(container.id).to.equal('1dcxxx000000034');
        chai_1.expect(container.success).to.be.equal(true);
        // tslint:disable-next-line:no-unused-expression
        chai_1.expect(container.errors).to.be.an('array').that.is.empty;
        chai_1.expect(mockToolingCreate.getCall(0).args[0]).to.equal('MetadataContainer');
        chai_1.expect(mockToolingCreate.getCall(0).args[1]).to.be.an('object');
    }));
    it('should throw an error when creating a metadata container fails', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        sandboxStub.stub(mockConnection.tooling, 'create').resolves({
            success: false,
            id: '',
            errors: ['Unexpected error while creating record'],
        });
        try {
            yield deployLibrary.createMetadataContainer();
            chai_1.expect.fail('Should have failed');
        }
        catch (e) {
            chai_1.expect(e.message).to.equal(i18n_1.nls.localize('beta_tapi_mdcontainer_error'));
            chai_1.expect(e.name).to.be.equal('DeployError');
        }
    }));
    it('should throw an error when creating a duplicate metadata container', () => __awaiter(void 0, void 0, void 0, function* () {
        const errorObj = {
            errorCode: 'DUPLICATE_VALUE',
            message: 'duplicate value found: Name duplicates value on record with id : 1dcxxx000000034',
            name: 'DUPLICATE_VALUE',
            stack: 'DUPLICATE_VALUE: duplicate value found: Name duplicates value on record with id : 1dcxxx000000034',
        };
        sandboxStub.stub(mockConnection.tooling, 'create').throws(errorObj);
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        try {
            yield deployLibrary.createMetadataContainer();
            chai_1.expect.fail('Should have failed');
        }
        catch (e) {
            chai_1.expect(e.message).to.equal('duplicate value found: Name duplicates value on record with id : 1dcxxx000000034');
            chai_1.expect(e.name).to.be.equal('DUPLICATE_VALUE');
        }
    }));
    it('should create a metadata member type for Apex Class', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        sandboxStub.stub(deployLibrary, 'getContentEntity').resolves(undefined);
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create').resolves({
            success: true,
            id: '400xxx000000034',
            errors: [],
        });
        deployLibrary.component = apexClassCmp;
        const containerMember = yield deployLibrary.createContainerMember(['file/path/one.cls', 'file/path/one.cls-meta.xml'], successfulContainerResult);
        chai_1.expect(containerMember.id).to.equal('400xxx000000034');
        chai_1.expect(containerMember.success).to.be.equal(true);
        // tslint:disable-next-line:no-unused-expression
        chai_1.expect(containerMember.errors).to.be.an('array').that.is.empty;
        chai_1.expect(mockToolingCreate.getCall(0).args[0]).to.equal('ApexClassMember');
    }));
    it('should create a metadata member type for Apex Trigger', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        sandboxStub.stub(deployLibrary, 'getContentEntity').resolves(undefined);
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create').resolves({
            success: true,
            id: '400xxx000000034',
            errors: [],
        });
        deployLibrary.component = apexTriggerCmp;
        const containerMember = yield deployLibrary.createContainerMember(['file/path/one.trigger', 'file/path/one.trigger-meta.xml'], successfulContainerResult);
        chai_1.expect(containerMember.id).to.equal('400xxx000000034');
        chai_1.expect(containerMember.success).to.be.equal(true);
        // tslint:disable-next-line:no-unused-expression
        chai_1.expect(containerMember.errors).to.be.an('array').that.is.empty;
        chai_1.expect(mockToolingCreate.getCall(0).args[0]).to.equal('ApexTriggerMember');
    }));
    it('should create a metadata member type for VisualForce Page', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        sandboxStub.stub(deployLibrary, 'getContentEntity').resolves(undefined);
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create').resolves({
            success: true,
            id: '400xxx000000034',
            errors: [],
        });
        deployLibrary.component = apexPageCmp;
        const containerMember = yield deployLibrary.createContainerMember(['file/path/one.page', 'file/path/one.page-meta.xml'], successfulContainerResult);
        chai_1.expect(containerMember.id).to.equal('400xxx000000034');
        chai_1.expect(containerMember.success).to.be.equal(true);
        // tslint:disable-next-line:no-unused-expression
        chai_1.expect(containerMember.errors).to.be.an('array').that.is.empty;
        chai_1.expect(mockToolingCreate.getCall(0).args[0]).to.equal('ApexPageMember');
    }));
    it('should create a metadata member type for VisualForce Component', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        sandboxStub.stub(deployLibrary, 'getContentEntity').resolves(undefined);
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create').resolves({
            success: true,
            id: '400xxx000000034',
            errors: [],
        });
        deployLibrary.component = apexComponent;
        const containerMember = yield deployLibrary.createContainerMember(['file/path/one.component', 'file/path/one.component-meta.xml'], successfulContainerResult);
        chai_1.expect(containerMember.id).to.equal('400xxx000000034');
        chai_1.expect(containerMember.success).to.be.equal(true);
        // tslint:disable-next-line:no-unused-expression
        chai_1.expect(containerMember.errors).to.be.an('array').that.is.empty;
        chai_1.expect(mockToolingCreate.getCall(0).args[0]).to.equal('ApexComponentMember');
    }));
    it('should call tooling api with the correct params when creating a metadata member type for new type', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        sandboxStub.stub(deployLibrary, 'getContentEntity').resolves(undefined);
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create').resolves({
            success: true,
            id: '400xxx000000034',
            errors: [],
        });
        deployLibrary.component = apexClassCmp;
        yield deployLibrary.createContainerMember(['file/path/one.cls', 'file/path/one.cls-meta.xml'], successfulContainerResult);
        chai_1.expect(mockToolingCreate.getCall(0).args[0]).to.be.equal('ApexClassMember');
        const secondParam = mockToolingCreate.getCall(0).args[1];
        chai_1.expect(secondParam).to.be.an('object');
        chai_1.expect(secondParam.MetadataContainerId).to.equal(successfulContainerResult.id);
        chai_1.expect(secondParam.FullName).to.equal('one');
        chai_1.expect(secondParam.Body).to.equal('public with sharing class TestAPI {}');
        chai_1.expect(secondParam.Metadata).to.deep.equal({
            apiVersion: '32.0',
            status: 'Active',
        });
        chai_1.expect(secondParam.contentEntityId).to.equal(undefined);
    }));
    it('should call tooling api with the correct params when creating a metadata member type for existing type', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        sandboxStub.stub(deployLibrary, 'getContentEntity').resolves('a00xxx000000034');
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create').resolves({
            success: true,
            id: '400xxx000000034',
            errors: [],
        });
        deployLibrary.component = apexClassCmp;
        yield deployLibrary.createContainerMember(['file/path/one.cls', 'file/path/one.cls-meta.xml'], successfulContainerResult);
        chai_1.expect(mockToolingCreate.getCall(0).args[0]).to.be.equal('ApexClassMember');
        const secondParam = mockToolingCreate.getCall(0).args[1];
        chai_1.expect(secondParam).to.be.an('object');
        chai_1.expect(secondParam.MetadataContainerId).to.equal(successfulContainerResult.id);
        chai_1.expect(secondParam.FullName).to.equal('one');
        chai_1.expect(secondParam.Body).to.equal('public with sharing class TestAPI {}');
        chai_1.expect(secondParam.Metadata).to.deep.equal({
            apiVersion: '32.0',
            status: 'Active',
        });
        chai_1.expect(secondParam.contentEntityId).to.equal('a00xxx000000034');
    }));
    it('should throw error when failing to create a metadata member type', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        sandboxStub.stub(deployLibrary, 'getContentEntity').resolves(undefined);
        sandboxStub.stub(mockConnection.tooling, 'create').resolves({
            success: false,
            id: '',
            errors: ['Unexpected error while creating record'],
        });
        deployLibrary.component = apexClassCmp;
        try {
            yield deployLibrary.createContainerMember(['file/path/one.cls', 'file/path/one.cls-meta.xml'], successfulContainerResult);
            chai_1.expect.fail('Should have failed');
        }
        catch (e) {
            chai_1.expect(e.message).to.equal(i18n_1.nls.localize('beta_tapi_membertype_error', 'ApexClass'));
            chai_1.expect(e.name).to.be.equal('DeployError');
        }
    }));
    it('should create a container async request', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create').resolves({
            success: true,
            id: '1drxxx000000034',
            errors: [],
        });
        const car = yield deployLibrary.createContainerAsyncRequest(successfulContainerResult);
        chai_1.expect(car.id).to.equal('1drxxx000000034');
        chai_1.expect(car.success).to.be.equal(true);
        // tslint:disable-next-line:no-unused-expression
        chai_1.expect(car.errors).to.be.an('array').that.is.empty;
        chai_1.expect(mockToolingCreate.getCall(0).args[0]).to.equal('ContainerAsyncRequest');
        chai_1.expect(mockToolingCreate.getCall(0).args[1]).to.deep.equal({
            MetadataContainerId: successfulContainerResult.id,
        });
    }));
    it('should throw an error when creating a container async request fails', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        sandboxStub.stub(mockConnection.tooling, 'create').resolves({
            success: false,
            id: '',
            errors: ['Unexpected error while creating record'],
        });
        try {
            yield deployLibrary.createContainerAsyncRequest(successfulContainerResult);
            chai_1.expect.fail('Should have failed');
        }
        catch (e) {
            chai_1.expect(e.message).to.equal(i18n_1.nls.localize('beta_tapi_car_error'));
            chai_1.expect(e.name).to.be.equal('DeployError');
        }
    }));
    it('should throw an error when creating a container async request', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        sandboxStub.stub(mockConnection.tooling, 'create').throwsException({
            message: 'insufficient access rights on cross-reference id: 1drxx000000xUHs',
            errorCode: 'INSUFFICIENT_ACCESS_ON_CROSS_REFERENCE_ENTITY',
            fields: [],
            name: 'INSUFFICIENT_ACCESS_ON_CROSS_REFERENCE_ENTITY',
        });
        try {
            yield deployLibrary.createContainerAsyncRequest(successfulContainerResult);
            chai_1.expect.fail('Should have failed');
        }
        catch (e) {
            chai_1.expect(e.message).to.equal('insufficient access rights on cross-reference id: 1drxx000000xUHs');
            chai_1.expect(e.name).to.be.equal('INSUFFICIENT_ACCESS_ON_CROSS_REFERENCE_ENTITY');
        }
    }));
    it('should poll for a container async request', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        deployLibrary.component = apexClassCmp;
        const mockToolingRetrieve = sandboxStub.stub(mockConnection.tooling, 'retrieve');
        mockToolingRetrieve.onCall(0).resolves({
            State: 'Queued',
            isDeleted: false,
            DeployDetails: null,
        });
        mockToolingRetrieve.onCall(1).resolves({
            State: "Completed" /* Completed */,
            isDeleted: false,
            DeployDetails: {
                componentFailures: [],
                componentSuccesses: [],
            },
        });
        const asyncRequestMock = {
            success: true,
            id: '1drxxx000000034',
            errors: [],
            name: 'TestCAR',
            message: '',
        };
        const pollCAR = yield deployLibrary.pollContainerStatus(asyncRequestMock.id);
        chai_1.expect(pollCAR.State).to.equal('Completed');
    }));
    it('should deploy successfully when namespace is defined', () => __awaiter(void 0, void 0, void 0, function* () {
        const deployLibrary = new deployStrategies_1.ContainerDeploy(mockConnection);
        // mock container creation
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create');
        mockToolingCreate.onCall(0).resolves({
            success: true,
            id: '1dcxxx000000034',
            errors: [],
        });
        // mock tooling query
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        mockToolingQuery.resolves({
            size: 1,
            totalSize: 1,
            done: true,
            queryLocator: '',
            entityTypeName: 'ApexClass',
            records: [{ Id: '00' }],
        });
        // mock container member creation
        mockToolingCreate.onCall(1).resolves({
            success: true,
            id: '400xxx000000034',
            errors: [],
        });
        // mock container async request creation
        mockToolingCreate.onCall(2).resolves({
            success: true,
            id: '1drxxx000000034',
            errors: [],
        });
        // mock status check
        sandboxStub.stub(mockConnection.tooling, 'retrieve').resolves({
            State: "Completed" /* Completed */,
            isDeleted: false,
            DeployDetails: {
                componentFailures: [],
                componentSuccesses: [
                    {
                        changed: true,
                        componentType: 'ApexComponent',
                        created: true,
                        createdDate: '2020-06-19T00:30:38.152+0000',
                        deleted: false,
                        fileName: 'component/one.component',
                        fullName: 'one',
                        id: '0992M000000uLGTQA2',
                        success: true,
                        warning: false,
                    },
                ],
            },
        });
        const result = yield deployLibrary.deploy(apexComponent, 't5tr');
        chai_1.expect(mockToolingCreate.calledThrice).to.be.true;
        chai_1.expect(mockToolingQuery.calledOnce).to.be.true;
        chai_1.expect(mockToolingQuery.calledWith(`Select Id from ApexComponent where Name = 'one' and NamespacePrefix = 't5tr'`)).to.be.true;
        chai_1.expect(result).to.deep.equals({
            id: undefined,
            status: "Completed" /* Completed */,
            success: true,
            components: [
                {
                    component: apexComponent,
                    diagnostics: [],
                    status: types_1.ComponentStatus.Changed,
                },
            ],
        });
    }));
});
//# sourceMappingURL=containerDeploy.test.js.map