"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const testSetup_1 = require("@salesforce/core/lib/testSetup");
const chai_1 = require("chai");
const fs = require("fs");
const path_1 = require("path");
const sinon_1 = require("sinon");
const i18n_1 = require("../../../src/i18n");
const deployStrategies_1 = require("../../../src/client/deployStrategies");
const metadata_registry_1 = require("../../../src/metadata-registry");
const client_1 = require("../../../src/client");
const $$ = testSetup_1.testSetup();
describe('LWC Deploy Strategy', () => {
    const testMetadataField = {
        apiVersion: '32.0',
        status: 'Active',
    };
    const testData = new testSetup_1.MockTestOrgData();
    let mockConnection;
    let sandboxStub;
    let simpleMetaXMLString = '<?xml version="1.0" encoding="UTF-8"?>';
    simpleMetaXMLString += '<ApexClass xmlns="http://soap.sforce.com/2006/04/metadata">';
    simpleMetaXMLString += '    <apiVersion>32.0</apiVersion>';
    simpleMetaXMLString += '    <status>Active</status>';
    simpleMetaXMLString += '</ApexClass>';
    const bundlePath = path_1.join('file', 'path', 'lwc', 'mockLwcCmp');
    const lwcFiles = [
        path_1.join(bundlePath, 'mockLwcCmp.js'),
        path_1.join(bundlePath, 'mockLwcCmp.html'),
        path_1.join(bundlePath, 'mockLwcCmp.js-meta.xml'),
    ];
    const lwcContents = [
        `import { LightningElement } from 'lwc';export default class TestLwc extends LightningElement {}`,
        `<template></template>`,
        simpleMetaXMLString,
    ];
    const tree = new metadata_registry_1.VirtualTreeContainer([
        {
            dirPath: bundlePath,
            children: lwcFiles.map((f) => path_1.basename(f)),
        },
    ]);
    const lwcComponent = new metadata_registry_1.SourceComponent({
        type: metadata_registry_1.registryData.types.lightningcomponentbundle,
        name: 'mockLwcCmp',
        content: bundlePath,
        xml: path_1.join(bundlePath, 'mockLwcCmp.js-meta.xml'),
    }, tree);
    const testLwcList = [
        {
            FilePath: lwcFiles[0],
            Format: 'js',
            Source: lwcContents[0],
            LightningComponentBundleId: '1dcxxx000000060',
        },
        {
            FilePath: lwcFiles[1],
            Format: 'html',
            Source: lwcContents[1],
            LightningComponentBundleId: '1dcxxx000000060',
        },
        {
            FilePath: lwcFiles[2],
            Format: 'js',
            Source: lwcContents[2],
            LightningComponentBundleId: '1dcxxx000000060',
        },
    ];
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
        $$.setConfigStubContents('AuthInfoConfig', {
            contents: yield testData.getConfig(),
        });
        mockConnection = yield core_1.Connection.create({
            authInfo: yield core_1.AuthInfo.create({
                username: testData.username,
            }),
        });
        const mockFS = sandboxStub.stub(fs, 'readFileSync');
        mockFS.withArgs(lwcFiles[0], 'utf8').returns(lwcContents[0]);
        mockFS.withArgs(lwcFiles[1], 'utf8').returns(lwcContents[1]);
        mockFS.withArgs(path_1.join(bundlePath, 'mockLwcCmp.js-meta.xml')).returns(simpleMetaXMLString);
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('should build list of lightning component resource objects with correct properties', () => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub
            .stub(mockConnection.tooling, 'query')
            // @ts-ignore
            .resolves({ records: [] });
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create');
        mockToolingCreate.resolves({
            success: true,
            id: '1dcxxx000000060',
            errors: [],
        });
        const lwcDeploy = new deployStrategies_1.LwcDeploy(mockConnection);
        lwcDeploy.component = lwcComponent;
        const lightningResources = yield lwcDeploy.buildResourceList();
        chai_1.expect(lightningResources).to.deep.include.members(testLwcList);
    }));
    it('should filter existing sources by filepath and attach id property for matches', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        const matches = [
            {
                Id: '1dcxxx000000034',
                Format: 'js',
                FilePath: lwcFiles[0],
                Source: lwcContents[0],
            },
            {
                Id: '1dcxxx000000035',
                Format: 'html',
                FilePath: lwcFiles[1],
                Source: lwcContents[1],
            },
        ];
        // @ts-ignore
        mockToolingQuery.resolves({ records: matches });
        sandboxStub.stub(deployStrategies_1.LwcDeploy.prototype, 'upsertBundle').resolves({
            success: true,
            id: '1dcxxx000000033',
            errors: [],
        });
        const lwcDeploy = new deployStrategies_1.LwcDeploy(mockConnection);
        lwcDeploy.component = lwcComponent;
        const lwcResults = yield lwcDeploy.buildResourceList();
        chai_1.expect(lwcResults).to.deep.include.members(matches);
    }));
    it('should filter existing sources and keep list unchanged if there are no matches', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        const matches = [
            {
                Id: '1dcxxx000000035',
                FilePath: path_1.join('path', 'to', 'wrong', 'lwc', 'lwcFile', 'lwcFile.html'),
                Format: 'html',
                Source: lwcContents[0],
                LightningComponentBundleId: '1dcxxx000000060',
            },
            {
                Id: '1dcxxx000000036',
                FilePath: 'path/to/wrong/lwc/lwcFile/lwcFile.js',
                Format: 'js',
                Source: lwcContents[1],
                LightningComponentBundleId: '1dcxxx000000060',
            },
        ];
        // @ts-ignore
        mockToolingQuery.resolves({ records: matches });
        sandboxStub.stub(deployStrategies_1.LwcDeploy.prototype, 'upsertBundle').resolves({
            success: true,
            id: '1dcxxx000000033',
            errors: [],
        });
        const lwcDeploy = new deployStrategies_1.LwcDeploy(mockConnection);
        lwcDeploy.component = lwcComponent;
        const lwcResults = yield lwcDeploy.buildResourceList();
        chai_1.expect(lwcResults).to.not.deep.include.members(matches);
    }));
    it('should create a lightningcomponentbundle given the fullname and metadata', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create');
        mockToolingCreate.resolves({
            success: true,
            id: '1dcxxx000000034',
            errors: [],
        });
        sandboxStub.stub(deployStrategies_1.LwcDeploy.prototype, 'buildMetadataField').returns(testMetadataField);
        const lwcDeploy = new deployStrategies_1.LwcDeploy(mockConnection);
        lwcDeploy.component = lwcComponent;
        const bundle = yield lwcDeploy.upsertBundle();
        chai_1.expect(bundle.id).to.equal('1dcxxx000000034');
        chai_1.expect(bundle.success).to.be.equal(true);
        // tslint:disable-next-line:no-unused-expression
        chai_1.expect(bundle.errors).to.be.an('array').that.is.empty;
        chai_1.expect(mockToolingCreate.getCall(0).args[0]).to.equal(lwcComponent.type.name);
        chai_1.expect(mockToolingCreate.getCall(0).args[1]).to.be.an('object');
    }));
    it('should update lightningcomponentbundle given the id and metadata', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockToolingUpdate = sandboxStub.stub(mockConnection.tooling, 'update');
        mockToolingUpdate.resolves({
            success: true,
            id: '1dcxxx000000034',
            errors: [],
        });
        sandboxStub.stub(deployStrategies_1.LwcDeploy.prototype, 'buildMetadataField').returns(testMetadataField);
        const lwcDeploy = new deployStrategies_1.LwcDeploy(mockConnection);
        lwcDeploy.component = lwcComponent;
        const bundle = yield lwcDeploy.upsertBundle('1dcxxx000000034');
        chai_1.expect(bundle.id).to.equal('1dcxxx000000034');
        chai_1.expect(bundle.success).to.be.equal(true);
        // tslint:disable-next-line:no-unused-expression
        chai_1.expect(bundle.errors).to.be.an('array').that.is.empty;
        chai_1.expect(mockToolingUpdate.getCall(0).args[0]).to.equal(lwcComponent.type.name);
        chai_1.expect(mockToolingUpdate.getCall(0).args[1]).to.be.an('object');
    }));
    it('should throw an error if there is a problem creating the bundle', () => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub.stub(mockConnection.tooling, 'create').resolves({
            success: false,
            id: '',
            errors: ['Unexpected error while creating record'],
        });
        sandboxStub.stub(deployStrategies_1.LwcDeploy.prototype, 'buildMetadataField').returns(testMetadataField);
        const lwcDeploy = new deployStrategies_1.LwcDeploy(mockConnection);
        lwcDeploy.component = lwcComponent;
        try {
            yield lwcDeploy.upsertBundle();
            chai_1.expect.fail('Should have failed');
        }
        catch (e) {
            chai_1.expect(e.message).to.equal(i18n_1.nls.localize('error_creating_metadata_type', 'LightningComponentBundle'));
            chai_1.expect(e.name).to.be.equal('DeployError');
        }
    }));
    it('should create sources in bundle and return successes in correct shape', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create');
        mockToolingCreate.resolves({
            success: true,
            id: '1dcxxx000000034',
            errors: [],
        });
        sandboxStub.stub(deployStrategies_1.LwcDeploy.prototype, 'buildMetadataField').returns(testMetadataField);
        sandboxStub.stub(deployStrategies_1.LwcDeploy.prototype, 'buildResourceList').resolves(testLwcList);
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        // @ts-ignore
        mockToolingQuery.resolves({ records: [] });
        const lwc = new deployStrategies_1.LwcDeploy(mockConnection);
        const result = yield lwc.deploy(lwcComponent, '');
        chai_1.expect(result).to.deep.equal({
            id: undefined,
            status: "Completed" /* Completed */,
            success: true,
            components: [
                {
                    component: lwcComponent,
                    status: client_1.ComponentStatus.Created,
                    diagnostics: [],
                },
            ],
        });
    }));
    it('should format output for creation only failures correctly', () => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub.stub(deployStrategies_1.LwcDeploy.prototype, 'buildResourceList').resolves(testLwcList);
        sandboxStub
            .stub(mockConnection.tooling, 'query')
            // @ts-ignore
            .resolves({ records: [] });
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create');
        mockToolingCreate.onFirstCall().resolves({
            success: true,
            id: '1dcxxx000000034',
            errors: [],
        });
        sandboxStub
            .stub(mockConnection.tooling, 'update')
            .throws(new Error('mockLwcCmp.js:1,1 : Unexpected error while creating sources'));
        const updateLwcList = [...testLwcList];
        updateLwcList.forEach((el) => {
            el.Id = '1dcxxx000000034';
            delete el.LightningComponentBundleId;
        });
        const lwcDeploy = new deployStrategies_1.LwcDeploy(mockConnection);
        const deployResult = yield lwcDeploy.deploy(lwcComponent, '');
        chai_1.expect(deployResult).to.deep.equal({
            id: undefined,
            status: "Failed" /* Failed */,
            success: false,
            components: [
                {
                    component: lwcComponent,
                    status: client_1.ComponentStatus.Failed,
                    diagnostics: [
                        {
                            columnNumber: 1,
                            lineNumber: 1,
                            filePath: lwcFiles[0],
                            message: 'Unexpected error while creating sources',
                            type: 'Error',
                        },
                        {
                            columnNumber: 1,
                            lineNumber: 1,
                            filePath: lwcFiles[0],
                            message: 'Unexpected error while creating sources',
                            type: 'Error',
                        },
                        {
                            columnNumber: 1,
                            lineNumber: 1,
                            filePath: lwcFiles[0],
                            message: 'Unexpected error while creating sources',
                            type: 'Error',
                        },
                    ],
                },
            ],
        });
    }));
});
//# sourceMappingURL=lwcDeploy.test.js.map