"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const diagnosticUtil_1 = require("../../src/client/diagnosticUtil");
const sourceComponent_1 = require("../../src/metadata-registry/sourceComponent");
const metadata_registry_1 = require("../../src/metadata-registry");
const path_1 = require("path");
const chai_1 = require("chai");
const types_1 = require("../../src/client/types");
function createDeployment(props, tree) {
    return {
        status: types_1.ComponentStatus.Failed,
        component: new sourceComponent_1.SourceComponent(props, tree),
        diagnostics: [],
    };
}
function createDeployMessage(props) {
    // @ts-ignore trading a bunch of ts-ignores for one ts-ignore
    return props;
}
describe('DiagnosticUtil', () => {
    describe('Default', () => {
        const util = new diagnosticUtil_1.DiagnosticUtil('metadata');
        const classes = path_1.join('path', 'to', 'classes');
        const component = {
            name: 'Test',
            type: metadata_registry_1.registryData.types.apexclass,
            content: path_1.join(classes, 'Test.cls'),
            xml: path_1.join(classes, 'Test.cls-meta.xml'),
        };
        const tree = new metadata_registry_1.VirtualTreeContainer([
            {
                dirPath: classes,
                children: ['Test.cls', 'Test.cls-meta.xml'],
            },
        ]);
        it('should create diagnostic for problem with message only', () => {
            const deployment = createDeployment(component, tree);
            const message = createDeployMessage({
                problem: 'This might be a problem later!',
                problemType: 'Warning',
            });
            chai_1.expect(util.setDiagnostic(deployment, message).diagnostics).to.deep.equal([
                {
                    message: 'This might be a problem later!',
                    type: 'Warning',
                },
            ]);
        });
        it('should create diagnostic for problem with file line and column info', () => {
            const deployment = createDeployment(component, tree);
            const message = createDeployMessage({
                problem: 'Expected a ;',
                problemType: 'Error',
                fileName: 'Test.cls',
                lineNumber: '4',
                columnNumber: '2',
            });
            chai_1.expect(util.setDiagnostic(deployment, message).diagnostics).to.deep.equal([
                {
                    message: 'Expected a ;',
                    type: 'Error',
                    filePath: component.content,
                    lineNumber: 4,
                    columnNumber: 2,
                },
            ]);
        });
    });
    describe('LWC', () => {
        const bundlePath = path_1.join('path', 'to', 'lwc', 'test');
        const component = {
            name: 'Test',
            type: metadata_registry_1.registryData.types.lightningcomponentbundle,
            xml: path_1.join(bundlePath, 'test.js-meta.xml'),
            content: bundlePath,
        };
        const tree = new metadata_registry_1.VirtualTreeContainer([
            {
                dirPath: bundlePath,
                children: ['test.js-meta.xml', 'test.js', 'test.html'],
            },
        ]);
        it('should create diagnostic for problem with message only using tooling api', () => {
            const util = new diagnosticUtil_1.DiagnosticUtil('tooling');
            const deployment = createDeployment(component, tree);
            const message = 'There was a problem with the component';
            chai_1.expect(util.setDiagnostic(deployment, message).diagnostics).to.deep.equal([
                {
                    message,
                    type: 'Error',
                },
            ]);
        });
        it('should create diagnostic for problem with file line and column info using tooling api', () => {
            const util = new diagnosticUtil_1.DiagnosticUtil('tooling');
            const deployment = createDeployment(component, tree);
            const message = 'Compilation Failure\n\ttest.html:3,12 : LWC1075: Multiple roots found';
            chai_1.expect(util.setDiagnostic(deployment, message).diagnostics).to.deep.equal([
                {
                    message: 'LWC1075: Multiple roots found',
                    type: 'Error',
                    filePath: path_1.join(bundlePath, 'test.html'),
                    lineNumber: 3,
                    columnNumber: 12,
                },
            ]);
        });
        it('should create diagnostic for problem with message only using metadata api', () => {
            const util = new diagnosticUtil_1.DiagnosticUtil('metadata');
            const deployment = createDeployment(component, tree);
            const message = createDeployMessage({
                problem: 'There was a problem deploying',
                problemType: 'Error',
            });
            chai_1.expect(util.setDiagnostic(deployment, message).diagnostics).to.deep.equal([
                {
                    message: message.problem,
                    type: message.problemType,
                },
            ]);
        });
        it('should create diagnostic for problem with file line and column info using metadata api', () => {
            const util = new diagnosticUtil_1.DiagnosticUtil('metadata');
            const deployment = createDeployment(component, tree);
            const message = createDeployMessage({
                problem: '[Line: 4, Col: 15] LWC1075: Multiple roots found',
                problemType: 'Error',
                fileName: path_1.join('test', 'test.html'),
            });
            chai_1.expect(util.setDiagnostic(deployment, message).diagnostics).to.deep.equal([
                {
                    message: 'LWC1075: Multiple roots found',
                    type: 'Error',
                    filePath: path_1.join(bundlePath, 'test.html'),
                    lineNumber: 4,
                    columnNumber: 15,
                },
            ]);
        });
    });
    describe('Aura', () => {
        const bundlePath = path_1.join('path', 'to', 'aura', 'test');
        const component = {
            name: 'Test',
            type: metadata_registry_1.registryData.types.auradefinitionbundle,
            xml: path_1.join(bundlePath, 'test.app-meta.xml'),
            content: bundlePath,
        };
        const tree = new metadata_registry_1.VirtualTreeContainer([
            {
                dirPath: bundlePath,
                children: ['test.app-meta.xml', 'test.app', 'testHelper.js', 'TestApp.auradoc'],
            },
        ]);
        it('should create diagnostic for problem with message only using metadata api', () => {
            const util = new diagnosticUtil_1.DiagnosticUtil('metadata');
            const deployment = createDeployment(component, tree);
            const message = createDeployMessage({
                problem: 'There was a problem deploying',
                problemType: 'Error',
            });
            chai_1.expect(util.setDiagnostic(deployment, message).diagnostics).to.deep.equal([
                {
                    message: message.problem,
                    type: message.problemType,
                },
            ]);
        });
        it('should create diagnostic for problem with file line and column info using metadata api', () => {
            const util = new diagnosticUtil_1.DiagnosticUtil('metadata');
            const deployment = createDeployment(component, tree);
            const message = createDeployMessage({
                problem: '[row,col]:[1,5]\nMessage: There was a typo',
                problemType: 'Error',
                fileName: path_1.join('test', 'testHelper.js'),
            });
            chai_1.expect(util.setDiagnostic(deployment, message).diagnostics).to.deep.equal([
                {
                    message: message.problem,
                    type: 'Error',
                    filePath: path_1.join(bundlePath, 'testHelper.js'),
                    lineNumber: 1,
                    columnNumber: 5,
                },
            ]);
        });
        it('should create diagnostic for problem with message only using tooling api', () => {
            const util = new diagnosticUtil_1.DiagnosticUtil('tooling');
            const deployment = createDeployment(component, tree);
            const message = 'There was a problem deploying';
            chai_1.expect(util.setDiagnostic(deployment, message).diagnostics).to.deep.equal([
                {
                    message,
                    type: 'Error',
                },
            ]);
        });
        it('should create diagnostic for problem with file line and column info using tooling api', () => {
            const util = new diagnosticUtil_1.DiagnosticUtil('tooling');
            const deployment = createDeployment(component, tree);
            const message = createDeployMessage({
                problem: "c.TestApp: Failed to parse HELPER for js://c.TestApp: Expected ',' or '}' [5, 1]: 's'",
                problemType: 'Error',
                fileName: path_1.join('test', 'testHelper.js'),
            });
            chai_1.expect(util.setDiagnostic(deployment, message).diagnostics).to.deep.equal([
                {
                    message: message.problem,
                    type: 'Error',
                    filePath: path_1.join(bundlePath, 'testHelper.js'),
                    lineNumber: 5,
                    columnNumber: 1,
                },
            ]);
        });
    });
});
//# sourceMappingURL=diagnosticUtil.test.js.map