"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const testSetup_1 = require("@salesforce/core/lib/testSetup");
const chai_1 = require("chai");
const fs = require("fs");
const path = require("path");
const stream = require("stream");
const sinon_1 = require("sinon");
const client_1 = require("../../src/client");
const metadata_registry_1 = require("../../src/metadata-registry");
const i18n_1 = require("../../src/i18n");
const assert_1 = require("assert");
const $$ = testSetup_1.testSetup();
describe('Tooling Retrieve', () => {
    const testData = new testSetup_1.MockTestOrgData();
    const registryAccess = new metadata_registry_1.RegistryAccess();
    let mockConnection;
    let sandboxStub;
    let metaXMLFile = '<?xml version="1.0" encoding="UTF-8"?>\n';
    metaXMLFile += '<ApexClass xmlns="http://soap.sforce.com/2006/04/metadata">\n';
    metaXMLFile += '\t<apiVersion>32.0</apiVersion>\n';
    metaXMLFile += '\t<status>Active</status>\n';
    metaXMLFile += '</ApexClass>';
    const mdComponents = [
        new metadata_registry_1.SourceComponent({
            type: {
                id: 'apexclass',
                name: 'ApexClass',
                directoryName: 'classes',
                inFolder: false,
                suffix: 'cls',
            },
            name: 'myTestClass',
            xml: path.join('file', 'path', 'myTestClass.cls-meta.xml'),
            content: path.join('file', 'path', 'myTestClass.cls'),
        }),
    ];
    const apexClassQueryResult = {
        done: true,
        entityTypeName: 'ApexClass',
        records: [
            {
                ApiVersion: '32',
                Body: 'public with sharing class myTestClass {}',
                Id: '01pxxx000000034',
                Name: 'myTestClass',
                NamespacePrefix: null,
                Status: 'Active',
            },
        ],
        size: 1,
        totalSize: 1,
        queryLocator: null,
    };
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
        $$.setConfigStubContents('AuthInfoConfig', {
            contents: yield testData.getConfig(),
        });
        mockConnection = yield core_1.Connection.create({
            authInfo: yield core_1.AuthInfo.create({
                username: testData.username,
            }),
        });
        sandboxStub.stub(fs, 'existsSync').returns(true);
        // @ts-ignore
        sandboxStub.stub(fs, 'lstatSync').returns({ isDirectory: () => false });
        const mockFS = sandboxStub.stub(fs, 'readFileSync');
        mockFS
            .withArgs(path.join('file', 'path', 'MyTestClass.cls'), 'utf8')
            .returns('public with sharing class TestAPI {}');
        mockFS
            .withArgs(path.join('file', 'path', 'MyTestClass.cls-meta.xml'), 'utf8')
            .returns(metaXMLFile);
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('should generate correct query to retrieve an ApexClass', () => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub.stub(registryAccess, 'getComponentsFromPath').returns(mdComponents);
        const toolingQueryStub = sandboxStub.stub(mockConnection.tooling, 'query');
        // @ts-ignore
        toolingQueryStub.returns(apexClassQueryResult);
        const stubCreateMetadataFile = sandboxStub.stub(fs, 'createWriteStream');
        sandboxStub.stub(fs, 'closeSync');
        sandboxStub.stub(fs, 'openSync');
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        stubCreateMetadataFile.onCall(0).returns(new stream.PassThrough());
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        stubCreateMetadataFile.onCall(1).returns(new stream.PassThrough());
        const toolingAPI = new client_1.ToolingApi(mockConnection, registryAccess);
        const retrieveOpts = {
            paths: [path.join('file', 'path', 'myTestClass.cls')],
            output: path.join('file', 'path'),
        };
        const retrieveResults = yield toolingAPI.retrieveWithPaths(retrieveOpts);
        chai_1.expect(retrieveResults).to.be.a('object');
        chai_1.expect(retrieveResults.success).to.equal(true);
        chai_1.expect(toolingQueryStub.calledOnceWith(`Select Id, ApiVersion, Body, Name, NamespacePrefix, Status from ApexClass where Name = 'myTestClass' and NamespacePrefix = ''`)).to.equal(true);
    }));
    it('should generate correct query to retrieve an ApexClass using namespace', () => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub.stub(registryAccess, 'getComponentsFromPath').returns(mdComponents);
        const toolingQueryStub = sandboxStub.stub(mockConnection.tooling, 'query');
        // @ts-ignore
        toolingQueryStub.returns(apexClassQueryResult);
        const stubCreateMetadataFile = sandboxStub.stub(fs, 'createWriteStream');
        sandboxStub.stub(fs, 'closeSync');
        sandboxStub.stub(fs, 'openSync');
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        stubCreateMetadataFile.onCall(0).returns(new stream.PassThrough());
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        stubCreateMetadataFile.onCall(1).returns(new stream.PassThrough());
        const toolingAPI = new client_1.ToolingApi(mockConnection, registryAccess);
        const retrieveOpts = {
            paths: [path.join('file', 'path', 'myTestClass.cls')],
            namespace: 'tstr',
            output: path.join('file', 'path'),
        };
        const retrieveResults = yield toolingAPI.retrieveWithPaths(retrieveOpts);
        chai_1.expect(retrieveResults).to.be.a('object');
        chai_1.expect(retrieveResults.success).to.equal(true);
        chai_1.expect(toolingQueryStub.calledOnceWith(`Select Id, ApiVersion, Body, Name, NamespacePrefix, Status from ApexClass where Name = 'myTestClass' and NamespacePrefix = 'tstr'`)).to.equal(true);
    }));
    it('should retrieve an ApexClass using filepath', () => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub.stub(registryAccess, 'getComponentsFromPath').returns(mdComponents);
        sandboxStub
            .stub(mockConnection.tooling, 'query')
            // @ts-ignore
            .returns(apexClassQueryResult);
        const stubCreateMetadataFile = sandboxStub.stub(fs, 'createWriteStream');
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        stubCreateMetadataFile.onCall(0).returns(new stream.PassThrough());
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        stubCreateMetadataFile.onCall(1).returns(new stream.PassThrough());
        sandboxStub.stub(fs, 'closeSync');
        sandboxStub.stub(fs, 'openSync');
        const toolingAPI = new client_1.ToolingApi(mockConnection, registryAccess);
        const retrieveOpts = {
            paths: [path.join('file', 'path', 'myTestClass.cls')],
            output: path.join('file', 'path'),
        };
        const retrieveResults = yield toolingAPI.retrieveWithPaths(retrieveOpts);
        chai_1.expect(retrieveResults).to.be.a('object');
        chai_1.expect(retrieveResults.success).to.equal(true);
        chai_1.expect(retrieveResults.components).to.be.a('Array');
        chai_1.expect(retrieveResults.components.length).to.equal(1);
        chai_1.expect(retrieveResults.components[0].fullName).to.equal('myTestClass');
        chai_1.expect(retrieveResults.components[0].type).to.be.a('object');
        chai_1.expect(retrieveResults.components[0].type.name).to.equal('ApexClass');
        chai_1.expect(retrieveResults.components[0].type.suffix).to.equal('cls');
        chai_1.expect(retrieveResults.components[0].type.directoryName).to.equal('classes');
        chai_1.expect(retrieveResults.components[0].type.inFolder).to.equal(false);
        chai_1.expect(retrieveResults.components[0].xml).to.equal(path.join('file', 'path', 'myTestClass.cls-meta.xml'));
        chai_1.expect(retrieveResults.components[0].walkContent().length).to.equal(1);
        chai_1.expect(retrieveResults.components[0].walkContent()[0]).to.equal(path.join('file', 'path', 'myTestClass.cls'));
    }));
    it('should retrieve an ApexClass using SourceComponents', () => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub
            .stub(mockConnection.tooling, 'query')
            // @ts-ignore
            .returns(apexClassQueryResult);
        const stubCreateMetadataFile = sandboxStub.stub(fs, 'createWriteStream');
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        stubCreateMetadataFile.onCall(0).returns(new stream.PassThrough());
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        stubCreateMetadataFile.onCall(1).returns(new stream.PassThrough());
        sandboxStub.stub(fs, 'closeSync');
        sandboxStub.stub(fs, 'openSync');
        const toolingAPI = new client_1.ToolingApi(mockConnection, registryAccess);
        const retrieveResults = yield toolingAPI.retrieve({
            components: mdComponents,
        });
        chai_1.expect(retrieveResults).to.be.a('object');
        chai_1.expect(retrieveResults.success).to.equal(true);
        chai_1.expect(retrieveResults.components).to.be.a('Array');
        chai_1.expect(retrieveResults.components.length).to.equal(1);
        chai_1.expect(retrieveResults.components[0].fullName).to.equal('myTestClass');
        chai_1.expect(retrieveResults.components[0].type).to.be.a('object');
        chai_1.expect(retrieveResults.components[0].type.name).to.equal('ApexClass');
        chai_1.expect(retrieveResults.components[0].type.suffix).to.equal('cls');
        chai_1.expect(retrieveResults.components[0].type.directoryName).to.equal('classes');
        chai_1.expect(retrieveResults.components[0].type.inFolder).to.equal(false);
        chai_1.expect(retrieveResults.components[0].xml).to.equal(path.join('file', 'path', 'myTestClass.cls-meta.xml'));
        chai_1.expect(retrieveResults.components[0].walkContent().length).to.equal(1);
        chai_1.expect(retrieveResults.components[0].walkContent()[0]).to.equal(path.join('file', 'path', 'myTestClass.cls'));
    }));
    it('should return empty result when metadata is not in org', () => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub.stub(registryAccess, 'getComponentsFromPath').returns(mdComponents);
        sandboxStub
            .stub(mockConnection.tooling, 'query')
            // @ts-ignore
            .returns({ done: true, entityTypeName: 'ApexClass', records: [] });
        const toolingAPI = new client_1.ToolingApi(mockConnection, registryAccess);
        const retrieveOpts = {
            paths: [path.join('file', 'path', 'myTestClass.cls')],
            output: path.join('file', 'path'),
        };
        const retrieveResults = yield toolingAPI.retrieveWithPaths(retrieveOpts);
        chai_1.expect(retrieveResults).to.be.a('object');
        chai_1.expect(retrieveResults.success).to.equal(true);
        chai_1.expect(retrieveResults.components).to.be.a('Array');
        chai_1.expect(retrieveResults.components.length).to.equal(0);
        chai_1.expect(retrieveResults.message).to.equal(i18n_1.nls.localize('error_md_not_present_in_org', 'myTestClass'));
    }));
    it('should throw an error when trying to retrieve more than one type at a time', () => __awaiter(void 0, void 0, void 0, function* () {
        mdComponents.push(new metadata_registry_1.SourceComponent({
            type: metadata_registry_1.registryData.types.apexclass,
            name: 'anotherClass',
            xml: path.join('file', 'path', 'anotherClass.cls-meta.xml'),
            content: path.join('file', 'path', 'anotherClass.cls'),
        }));
        const toolingAPI = new client_1.ToolingApi(mockConnection, registryAccess);
        try {
            yield toolingAPI.retrieve({
                components: mdComponents,
            });
            assert_1.fail('Retrieve should have thrown an error');
        }
        catch (e) {
            chai_1.expect(e.message).to.equals(i18n_1.nls.localize('tapi_retrieve_component_limit_error'));
            chai_1.expect(e.name).to.equals('MetadataRetrieveLimit');
        }
    }));
    it('should throw an error when trying to retrieve an unsupported type', () => __awaiter(void 0, void 0, void 0, function* () {
        const unsupportedComponent = [
            new metadata_registry_1.SourceComponent({
                type: {
                    id: 'fancytype',
                    name: 'FancyType',
                    directoryName: 'fancy',
                    inFolder: false,
                    suffix: 'b',
                },
                name: 'anotherOne',
                xml: path.join('file', 'path', 'anotherOne.b-meta.xml'),
                content: path.join('file', 'path', 'anotherOne.b'),
            }),
        ];
        const toolingAPI = new client_1.ToolingApi(mockConnection, registryAccess);
        try {
            yield toolingAPI.retrieve({
                components: unsupportedComponent,
            });
            assert_1.fail('Retrieve should have thrown an error');
        }
        catch (e) {
            chai_1.expect(e.message).to.equals(i18n_1.nls.localize('beta_tapi_membertype_unsupported_error', 'FancyType'));
            chai_1.expect(e.name).to.equals('MetadataTypeUnsupported');
        }
    }));
});
//# sourceMappingURL=retrieve.test.js.map