"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const path_1 = require("path");
const sinon_1 = require("sinon");
const convertTransaction_1 = require("../../src/convert/convertTransaction");
const streams_1 = require("../../src/convert/streams");
const constants_1 = require("../../src/utils/constants");
const registry_1 = require("../mock/registry");
const finalizers_1 = require("../mock/convert/finalizers");
const env = sinon_1.createSandbox();
describe('Convert Transaction Constructs', () => {
    afterEach(() => env.restore());
    describe('ConvertTransaction', () => {
        it('should only set a finalizer once per transaction', () => {
            const transaction = new convertTransaction_1.ConvertTransaction();
            // @ts-ignore visible for testing
            const setSpy = env.spy(transaction.finalizers, 'set');
            transaction.addFinalizer(finalizers_1.TestFinalizerNoWrites);
            transaction.addFinalizer(finalizers_1.TestFinalizerNoWrites);
            chai_1.expect(setSpy.calledOnce).to.be.true;
            env.restore();
        });
        it('should yield results of finalizers upon executeFinalizers', () => __awaiter(void 0, void 0, void 0, function* () {
            var e_1, _a;
            const transaction = new convertTransaction_1.ConvertTransaction();
            transaction.addFinalizer(finalizers_1.TestFinalizerNoWrites);
            transaction.addFinalizer(finalizers_1.TestFinalizerNoResult);
            const results = [];
            try {
                for (var _b = __asyncValues(transaction.executeFinalizers()), _c; _c = yield _b.next(), !_c.done;) {
                    const result = _c.value;
                    results.push(Array.isArray(result) ? result : result);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            chai_1.expect(results).to.deep.equal([
                {
                    component: registry_1.keanu.KEANU_COMPONENT,
                    writeInfos: [],
                },
                undefined,
            ]);
        }));
    });
    describe('RecompositionFinalizer', () => {
        it('should return a WriterFormat with recomposed data', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = registry_1.regina.REGINA_COMPONENT;
            const children = component.getChildren();
            const readStub = env.stub(component.tree, 'readFile');
            readStub.withArgs(children[0].xml).resolves(Buffer.from('<Y><test>child1</test></Y>'));
            readStub.withArgs(children[1].xml).resolves(Buffer.from('<X><test>child2</test></X>'));
            const finalizer = new convertTransaction_1.RecompositionFinalizer();
            const state = {
                recompose: {
                    Test__c: {
                        component,
                        children,
                    },
                },
            };
            const result = yield finalizer.finalize(state);
            chai_1.expect(result).to.deep.equal([
                {
                    component: component,
                    writeInfos: [
                        {
                            source: new streams_1.JsToXml({
                                ReginaKing: {
                                    [constants_1.XML_NS_KEY]: constants_1.XML_NS,
                                    ys: [{ test: 'child1' }],
                                    xs: [{ test: 'child2' }],
                                },
                            }),
                            relativeDestination: path_1.join('reginas', 'a.regina'),
                        },
                    ],
                },
            ]);
        }));
    });
});
//# sourceMappingURL=convertTransaction.test.js.map