"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const convert_1 = require("../../src/convert");
const sinon_1 = require("sinon");
const registry_1 = require("../mock/registry");
const metadata_registry_1 = require("../../src/metadata-registry");
const streams = require("../../src/convert/streams");
const fs = require("fs");
const fsUtil = require("../../src/utils/fileSystemHandler");
const path_1 = require("path");
const chai_1 = require("chai");
const constants_1 = require("../../src/utils/constants");
const errors_1 = require("../../src/errors");
const env = sinon_1.createSandbox();
describe('MetadataConverter', () => {
    let ensureDirectoryStub;
    let pipelineStub;
    let writeFileStub;
    const mockRegistryAccess = new metadata_registry_1.RegistryAccess(registry_1.mockRegistry);
    const converter = new convert_1.MetadataConverter(mockRegistryAccess);
    const components = registry_1.kathy.KATHY_COMPONENTS;
    const packageName = 'test';
    const outputDirectory = path_1.join('path', 'to', 'output');
    const packageOutput = path_1.join(outputDirectory, packageName);
    /* eslint-disable-next-line  @typescript-eslint/no-explicit-any */
    function validatePipelineArgs(pipelineArgs) {
        chai_1.expect(pipelineArgs[0] instanceof streams.ComponentReader).to.be.true;
        chai_1.expect(pipelineArgs[0].components).to.deep.equal(components);
        chai_1.expect(pipelineArgs[1] instanceof streams.ComponentConverter).to.be.true;
        chai_1.expect(pipelineArgs[1].targetFormat).to.equal('metadata');
        chai_1.expect(pipelineArgs[2] instanceof streams.ComponentWriter).to.be.true;
    }
    beforeEach(() => {
        ensureDirectoryStub = env.stub(fsUtil, 'ensureDirectoryExists');
        pipelineStub = env.stub(streams, 'pipeline').resolves();
        writeFileStub = env.stub(fs.promises, 'writeFile').resolves();
        env.stub(fs, 'createWriteStream');
    });
    afterEach(() => env.restore());
    it('should initialize with default RegistryAccess by default', () => {
        const defaultConverter = new convert_1.MetadataConverter();
        // @ts-ignore registryAccess private
        chai_1.expect(defaultConverter.registryAccess.registry).to.deep.equal(metadata_registry_1.registryData);
    });
    it('should generate package name using timestamp when option omitted', () => __awaiter(void 0, void 0, void 0, function* () {
        const timestamp = 123456;
        const packagePath = path_1.join(outputDirectory, `${constants_1.DEFAULT_PACKAGE_PREFIX}_${timestamp}`);
        env.stub(Date, 'now').returns(timestamp);
        yield converter.convert(components, 'metadata', {
            type: 'directory',
            outputDirectory,
        });
        chai_1.expect(pipelineStub.firstCall.args[2].rootDestination).to.equal(packagePath);
    }));
    it('should throw ConversionError when an error occurs', () => __awaiter(void 0, void 0, void 0, function* () {
        const error = new Error('whoops!');
        const expectedError = new errors_1.ConversionError(error);
        pipelineStub.rejects(error);
        try {
            yield converter.convert(components, 'metadata', {
                type: 'directory',
                outputDirectory,
            });
            chai_1.assert.fail('an error should have been thrown');
        }
        catch (e) {
            chai_1.expect(e.message).to.equal(expectedError.message);
            chai_1.expect(e.name).to.equal(expectedError.name);
        }
    }));
    describe('Directory Output', () => {
        it('should ensure directory exists before starting conversion', () => __awaiter(void 0, void 0, void 0, function* () {
            yield converter.convert(components, 'metadata', {
                type: 'directory',
                outputDirectory,
                packageName,
            });
            chai_1.expect(ensureDirectoryStub.calledBefore(pipelineStub)).to.be.true;
            chai_1.expect(ensureDirectoryStub.firstCall.args[0]).to.equal(packageOutput);
        }));
        it('should create conversion pipeline with proper stream configuration', () => __awaiter(void 0, void 0, void 0, function* () {
            yield converter.convert(components, 'metadata', {
                type: 'directory',
                outputDirectory,
                packageName,
            });
            const pipelineArgs = pipelineStub.firstCall.args;
            validatePipelineArgs(pipelineArgs);
            chai_1.expect(pipelineArgs[2] instanceof streams.StandardWriter).to.be.true;
            chai_1.expect(pipelineArgs[2].rootDestination).to.equal(packageOutput);
        }));
        it('should return packagePath in result', () => __awaiter(void 0, void 0, void 0, function* () {
            const result = yield converter.convert(components, 'metadata', {
                type: 'directory',
                outputDirectory,
                packageName,
            });
            chai_1.expect(result.packagePath).to.equal(packageOutput);
        }));
        it('should write manifest for metadata format conversion', () => __awaiter(void 0, void 0, void 0, function* () {
            const timestamp = 123456;
            const packagePath = path_1.join(outputDirectory, `${constants_1.DEFAULT_PACKAGE_PREFIX}_${timestamp}`);
            env.stub(Date, 'now').returns(timestamp);
            const expectedContents = new metadata_registry_1.ManifestGenerator(mockRegistryAccess).createManifest(components);
            yield converter.convert(components, 'metadata', { type: 'directory', outputDirectory });
            chai_1.expect(writeFileStub.calledBefore(pipelineStub)).to.be.true;
            chai_1.expect(writeFileStub.firstCall.args).to.deep.equal([
                path_1.join(packagePath, constants_1.PACKAGE_XML_FILE),
                expectedContents,
            ]);
        }));
        it('should not write manifest for source format conversion', () => __awaiter(void 0, void 0, void 0, function* () {
            yield converter.convert(components, 'source', { type: 'directory', outputDirectory });
            chai_1.expect(writeFileStub.notCalled).to.be.true;
        }));
    });
    describe('Zip Output', () => {
        it('should ensure directory exists before starting conversion', () => __awaiter(void 0, void 0, void 0, function* () {
            const zipPath = packageOutput + '.zip';
            yield converter.convert(components, 'metadata', {
                type: 'zip',
                outputDirectory,
                packageName,
            });
            chai_1.expect(ensureDirectoryStub.calledBefore(pipelineStub)).to.be.true;
            chai_1.expect(ensureDirectoryStub.firstCall.args[0]).to.equal(path_1.dirname(zipPath));
        }));
        it('should create conversion pipeline with fs write configuration', () => __awaiter(void 0, void 0, void 0, function* () {
            yield converter.convert(components, 'metadata', {
                type: 'zip',
                outputDirectory,
                packageName,
            });
            // secondCall is used because ZipWriter uses pipeline upon construction
            const pipelineArgs = pipelineStub.secondCall.args;
            validatePipelineArgs(pipelineArgs);
            chai_1.expect(pipelineArgs[2] instanceof streams.ZipWriter).to.be.true;
            chai_1.expect(pipelineArgs[2].rootDestination).to.equal(`${packageOutput}.zip`);
        }));
        it('should create conversion pipeline with in-memory configuration', () => __awaiter(void 0, void 0, void 0, function* () {
            yield converter.convert(components, 'metadata', { type: 'zip' });
            const pipelineArgs = pipelineStub.secondCall.args;
            validatePipelineArgs(pipelineArgs);
            chai_1.expect(pipelineArgs[2] instanceof streams.ZipWriter).to.be.true;
            chai_1.expect(pipelineArgs[2].rootDestination).to.be.undefined;
        }));
        it('should return zipBuffer result for in-memory configuration', () => __awaiter(void 0, void 0, void 0, function* () {
            const testBuffer = Buffer.from('oh hi mark');
            env.stub(streams.ZipWriter.prototype, 'buffer').value(testBuffer);
            const result = yield converter.convert(components, 'metadata', { type: 'zip' });
            chai_1.expect(result.zipBuffer).to.deep.equal(testBuffer);
        }));
        it('should return packagePath in result', () => __awaiter(void 0, void 0, void 0, function* () {
            const result = yield converter.convert(components, 'metadata', {
                type: 'zip',
                outputDirectory,
                packageName,
            });
            chai_1.expect(result.packagePath).to.equal(`${packageOutput}.zip`);
        }));
        it('should write manifest for metadata format conversion', () => __awaiter(void 0, void 0, void 0, function* () {
            const expectedContents = new metadata_registry_1.ManifestGenerator(mockRegistryAccess).createManifest(components);
            const addToZipStub = env.stub(streams.ZipWriter.prototype, 'addToZip');
            yield converter.convert(components, 'metadata', { type: 'zip' });
            chai_1.expect(addToZipStub.calledBefore(pipelineStub)).to.be.true;
            chai_1.expect(addToZipStub.firstCall.args).to.deep.equal([expectedContents, constants_1.PACKAGE_XML_FILE]);
        }));
        it('should not write manifest for source format conversion', () => __awaiter(void 0, void 0, void 0, function* () {
            const addToZipStub = env.stub(streams.ZipWriter.prototype, 'addToZip');
            yield converter.convert(components, 'source', { type: 'zip' });
            chai_1.expect(addToZipStub.notCalled).to.be.true;
        }));
    });
});
//# sourceMappingURL=metadataConverter.test.js.map