"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const registry_1 = require("../../mock/registry");
const decomposedMetadataTransformer_1 = require("../../../src/convert/transformers/decomposedMetadataTransformer");
const convertTransaction_1 = require("../../../src/convert/convertTransaction");
const chai_1 = require("chai");
const sinon_1 = require("sinon");
const constants_1 = require("../../../src/utils/constants");
const path_1 = require("path");
const streams_1 = require("../../../src/convert/streams");
const decomposedTopLevelConstants_1 = require("../../mock/registry/decomposedTopLevelConstants");
const env = sinon_1.createSandbox();
describe('DecomposedMetadataTransformer', () => {
    const component = registry_1.regina.REGINA_COMPONENT;
    const composedXmlObj = {
        ReginaKing: {
            [constants_1.XML_NS_KEY]: constants_1.XML_NS,
            fullName: component.fullName,
            foo: 'bar',
            ys: [{ fullName: 'child', test: 'testVal' }],
            xs: [{ fullName: 'child2', test: 'testVal2' }],
        },
    };
    afterEach(() => env.restore());
    it('should register RecompositionFinalizer', () => {
        const transaction = new convertTransaction_1.ConvertTransaction();
        const addFinalizerSpy = env.spy(transaction, 'addFinalizer');
        new decomposedMetadataTransformer_1.DecomposedMetadataTransformer(registry_1.mockRegistry, transaction);
        chai_1.expect(addFinalizerSpy.calledWith(convertTransaction_1.RecompositionFinalizer)).to.be.true;
    });
    describe('toMetadataFormat', () => {
        it('should delay for partial recomposition when a child component is given', () => __awaiter(void 0, void 0, void 0, function* () {
            const child = component.getChildren()[0];
            const transaction = new convertTransaction_1.ConvertTransaction();
            const transformer = new decomposedMetadataTransformer_1.DecomposedMetadataTransformer(registry_1.mockRegistry, transaction);
            const writerFormat = yield transformer.toMetadataFormat(child);
            chai_1.expect(writerFormat).to.deep.equal({ component: child, writeInfos: [] });
            chai_1.expect(transaction.state).to.deep.equal({
                recompose: {
                    [component.fullName]: {
                        component,
                        children: [child],
                    },
                },
            });
        }));
        it('should fully recompose metadata when a parent component is given', () => __awaiter(void 0, void 0, void 0, function* () {
            const transformer = new decomposedMetadataTransformer_1.DecomposedMetadataTransformer(registry_1.mockRegistry, new convertTransaction_1.ConvertTransaction());
            const children = component.getChildren();
            env.stub(component, 'parseXml').resolves({
                ReginaKing: {
                    [constants_1.XML_NS_KEY]: constants_1.XML_NS,
                    fullName: component.fullName,
                    foo: 'bar',
                },
            });
            env.stub(children[0], 'parseXml').resolves({
                Y: {
                    fullName: 'child',
                    test: 'testVal',
                },
            });
            env.stub(children[1], 'parseXml').resolves({
                X: {
                    fullName: 'child2',
                    test: 'testVal2',
                },
            });
            // force getChildren to return the children we just stubbed
            env.stub(component, 'getChildren').returns(children);
            const result = yield transformer.toMetadataFormat(component);
            chai_1.expect(result).to.deep.equal({
                component,
                writeInfos: [
                    {
                        source: new streams_1.JsToXml(composedXmlObj),
                        relativeDestination: path_1.join('reginas', 'a.regina'),
                    },
                ],
            });
        }));
    });
    describe('toSourceFormat', () => {
        it('should decompose children into respective files for "topLevel" config', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = decomposedTopLevelConstants_1.DECOMPOSED_TOP_LEVEL_COMPONENT;
            const { fullName, type } = component;
            const transformer = new decomposedMetadataTransformer_1.DecomposedMetadataTransformer(registry_1.mockRegistry);
            const root = path_1.join('main', 'default', type.directoryName, fullName);
            env.stub(component, 'parseXml').resolves({
                DecomposedTopLevel: {
                    [constants_1.XML_NS_KEY]: constants_1.XML_NS,
                    fullName,
                    foo: 'bar',
                    gs: [
                        { name: 'child', test: 'testVal' },
                        { name: 'child2', test: 'testVal2' },
                    ],
                },
            });
            const result = yield transformer.toSourceFormat(component);
            chai_1.expect(result).to.deep.equal({
                component,
                writeInfos: [
                    {
                        source: new streams_1.JsToXml({
                            G: {
                                [constants_1.XML_NS_KEY]: constants_1.XML_NS,
                                name: 'child',
                                test: 'testVal',
                            },
                        }),
                        relativeDestination: path_1.join(root, 'child.g-meta.xml'),
                    },
                    {
                        source: new streams_1.JsToXml({
                            G: {
                                [constants_1.XML_NS_KEY]: constants_1.XML_NS,
                                name: 'child2',
                                test: 'testVal2',
                            },
                        }),
                        relativeDestination: path_1.join(root, 'child2.g-meta.xml'),
                    },
                    {
                        source: new streams_1.JsToXml({
                            DecomposedTopLevel: {
                                [constants_1.XML_NS_KEY]: constants_1.XML_NS,
                                fullName,
                                foo: 'bar',
                            },
                        }),
                        relativeDestination: path_1.join(root, `${fullName}.${type.suffix}-meta.xml`),
                    },
                ],
            });
        }));
        it('should decompose children into respective directories and files for "folderPerType" config', () => __awaiter(void 0, void 0, void 0, function* () {
            const { type, fullName } = component;
            const transformer = new decomposedMetadataTransformer_1.DecomposedMetadataTransformer(registry_1.mockRegistry);
            const root = path_1.join('main', 'default', type.directoryName, fullName);
            env.stub(component, 'parseXml').resolves({
                ReginaKing: {
                    [constants_1.XML_NS_KEY]: constants_1.XML_NS,
                    fullName,
                    foo: 'bar',
                    ys: { fullName: 'child', test: 'testVal' },
                    xs: [
                        { fullName: 'child2', test: 'testVal2' },
                        { fullName: 'child3', test: 'testVal3' },
                    ],
                },
            });
            const result = yield transformer.toSourceFormat(component);
            chai_1.expect(result).to.deep.equal({
                component,
                writeInfos: [
                    {
                        source: new streams_1.JsToXml({
                            Y: {
                                [constants_1.XML_NS_KEY]: constants_1.XML_NS,
                                fullName: 'child',
                                test: 'testVal',
                            },
                        }),
                        relativeDestination: path_1.join(root, type.children.types.y.directoryName, 'child.y-meta.xml'),
                    },
                    {
                        source: new streams_1.JsToXml({
                            X: {
                                [constants_1.XML_NS_KEY]: constants_1.XML_NS,
                                fullName: 'child2',
                                test: 'testVal2',
                            },
                        }),
                        relativeDestination: path_1.join(root, type.children.types.x.directoryName, 'child2.x-meta.xml'),
                    },
                    {
                        source: new streams_1.JsToXml({
                            X: {
                                [constants_1.XML_NS_KEY]: constants_1.XML_NS,
                                fullName: 'child3',
                                test: 'testVal3',
                            },
                        }),
                        relativeDestination: path_1.join(root, type.children.types.x.directoryName, 'child3.x-meta.xml'),
                    },
                    {
                        source: new streams_1.JsToXml({
                            ReginaKing: {
                                [constants_1.XML_NS_KEY]: constants_1.XML_NS,
                                fullName: component.fullName,
                                foo: 'bar',
                            },
                        }),
                        relativeDestination: path_1.join(root, `${fullName}.${type.suffix}-meta.xml`),
                    },
                ],
            });
        }));
    });
});
//# sourceMappingURL=decomposedMetadataTransformer.js.map