"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const path_1 = require("path");
const registry_1 = require("../../mock/registry");
const defaultSourceAdapter_1 = require("../../../src/metadata-registry/adapters/defaultSourceAdapter");
const chai_1 = require("chai");
const baseSourceAdapter_1 = require("../../../src/metadata-registry/adapters/baseSourceAdapter");
const errors_1 = require("../../../src/errors");
const i18n_1 = require("../../../src/i18n");
const registryTestUtil_1 = require("../registryTestUtil");
const metadata_registry_1 = require("../../../src/metadata-registry");
class TestChildAdapter extends baseSourceAdapter_1.BaseSourceAdapter {
    getRootMetadataXmlPath() {
        return TestChildAdapter.xmlPath;
    }
    populate(trigger, component) {
        component.content = trigger;
        return component;
    }
}
TestChildAdapter.xmlPath = path_1.join('path', 'to', 'dwaynes', 'a.dwayne-meta.xml');
describe('BaseSourceAdapter', () => {
    it('Should reformat the fullName for folder types', () => {
        const path = path_1.join('path', 'to', 'kathys', 'A_Folder', 'My_Test.kathy-meta.xml');
        const type = registry_1.mockRegistry.types.kathybates;
        const adapter = new defaultSourceAdapter_1.DefaultSourceAdapter(type, registry_1.mockRegistry);
        chai_1.expect(adapter.getComponent(path)).to.deep.equal(new metadata_registry_1.SourceComponent({
            name: 'A_Folder/My_Test',
            type,
            xml: path,
        }));
    });
    it('Should defer parsing metadata xml to child adapter if path is not a metadata xml', () => {
        const path = path_1.join('path', 'to', 'dwaynes', 'My_Test.js');
        const type = registry_1.mockRegistry.types.dwaynejohnson;
        const adapter = new TestChildAdapter(type, registry_1.mockRegistry);
        chai_1.expect(adapter.getComponent(path)).to.deep.equal(new metadata_registry_1.SourceComponent({
            name: 'a',
            type,
            xml: TestChildAdapter.xmlPath,
            content: path,
        }));
    });
    it('Should defer parsing metadata xml to child adapter if path is not a root metadata xml', () => {
        const path = path_1.join('path', 'to', 'dwaynes', 'smallDwaynes', 'b.small-meta.xml');
        const type = registry_1.mockRegistry.types.dwaynejohnson;
        const adapter = new TestChildAdapter(type, registry_1.mockRegistry);
        chai_1.expect(adapter.getComponent(path)).to.deep.equal(new metadata_registry_1.SourceComponent({
            name: 'a',
            type,
            xml: TestChildAdapter.xmlPath,
            content: path,
        }));
    });
    it('Should throw an error if a metadata xml file is forceignored', () => {
        const testUtil = new registryTestUtil_1.RegistryTestUtil();
        const path = path_1.join('path', 'to', 'keanus', 'My_Test.keanu-meta.xml');
        const type = registry_1.mockRegistry.types.keanureeves;
        const forceIgnore = testUtil.stubForceIgnore({
            seed: path,
            deny: [path],
        });
        const adapter = new TestChildAdapter(type, registry_1.mockRegistry, forceIgnore);
        chai_1.assert.throws(() => adapter.getComponent(path), errors_1.UnexpectedForceIgnore, i18n_1.nls.localize('error_no_metadata_xml_ignore', [path, path]));
        testUtil.restore();
    });
    it('Should resolve a folder component', () => {
        const path = path_1.join('path', 'to', 'seans', 'My_Test-meta.xml');
        const type = registry_1.mockRegistry.types.seanfolder;
        const adapter = new defaultSourceAdapter_1.DefaultSourceAdapter(type, registry_1.mockRegistry);
        chai_1.expect(adapter.getComponent(path)).to.deep.equal(new metadata_registry_1.SourceComponent({
            name: 'My_Test',
            type,
            xml: path,
        }));
    });
    it('Should not recognize a content metadata file in the wrong directory', () => {
        const path = path_1.join('path', 'to', 'genes', 'My_Test.sean');
        const type = registry_1.mockRegistry.types.seanconnerys;
        const adapter = new defaultSourceAdapter_1.DefaultSourceAdapter(type, registry_1.mockRegistry);
        chai_1.expect(adapter.getComponent(path)).to.be.undefined;
    });
});
//# sourceMappingURL=baseSourceAdapter.js.map