"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const matchingContentSourceAdapter_1 = require("../../../src/metadata-registry/adapters/matchingContentSourceAdapter");
const registry_1 = require("../../mock/registry");
const chai_1 = require("chai");
const errors_1 = require("../../../src/errors");
const path_1 = require("path");
const registryTestUtil_1 = require("../registryTestUtil");
const i18n_1 = require("../../../src/i18n");
const treeContainers_1 = require("../../../src/metadata-registry/treeContainers");
const keanuConstants_1 = require("../../mock/registry/keanuConstants");
const metadata_registry_1 = require("../../../src/metadata-registry");
describe('MatchingContentSourceAdapter', () => {
    const type = registry_1.mockRegistry.types.keanureeves;
    const { KEANU_SOURCE_PATHS, KEANU_XML_PATHS, KEANU_COMPONENT, KEANUS_DIR } = registry_1.keanu;
    const tree = new treeContainers_1.VirtualTreeContainer([
        {
            dirPath: KEANUS_DIR,
            children: [keanuConstants_1.KEANU_SOURCE_NAMES[0], keanuConstants_1.KEANU_XML_NAMES[0]],
        },
    ]);
    const expectedComponent = new metadata_registry_1.SourceComponent(KEANU_COMPONENT, tree);
    const adapter = new matchingContentSourceAdapter_1.MatchingContentSourceAdapter(type, registry_1.mockRegistry, undefined, tree);
    it('Should return expected SourceComponent when given a root metadata xml path', () => {
        chai_1.expect(adapter.getComponent(KEANU_XML_PATHS[0])).to.deep.equal(expectedComponent);
    });
    it('Should return expected SourceComponent when given a source path', () => {
        chai_1.expect(adapter.getComponent(KEANU_SOURCE_PATHS[0])).to.deep.equal(expectedComponent);
    });
    it('Should throw an ExpectedSourceFilesError if no source is found from xml', () => {
        const path = path_1.join(KEANUS_DIR, 'b.xyz-meta.xml');
        chai_1.assert.throws(() => adapter.getComponent(path), errors_1.ExpectedSourceFilesError);
    });
    it('Should throw an ExpectedSourceFilesError if source and suffix not found', () => {
        const path = path_1.join(KEANUS_DIR, 'b.xyz');
        chai_1.assert.throws(() => adapter.getComponent(path), errors_1.ExpectedSourceFilesError);
    });
    it('Should throw an error if content file is forceignored', () => {
        const testUtil = new registryTestUtil_1.RegistryTestUtil();
        const path = KEANU_SOURCE_PATHS[0];
        const forceIgnore = testUtil.stubForceIgnore({
            seed: KEANU_XML_PATHS[0],
            deny: [path],
        });
        const adapter = new matchingContentSourceAdapter_1.MatchingContentSourceAdapter(type, registry_1.mockRegistry, forceIgnore, tree);
        chai_1.assert.throws(() => adapter.getComponent(path), errors_1.UnexpectedForceIgnore, i18n_1.nls.localize('error_no_source_ignore', [type.name, path]));
        testUtil.restore();
    });
});
//# sourceMappingURL=matchingContentSourceAdapter.js.map