"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const registry_1 = require("../../mock/registry");
const chai_1 = require("chai");
const mixedContentSourceAdapter_1 = require("../../../src/metadata-registry/adapters/mixedContentSourceAdapter");
const errors_1 = require("../../../src/errors");
const treeContainers_1 = require("../../../src/metadata-registry/treeContainers");
const metadata_registry_1 = require("../../../src/metadata-registry");
const tarajiConstants_1 = require("../../mock/registry/tarajiConstants");
describe('MixedContentSourceAdapter', () => {
    it('Should throw ExpectedSourceFilesError if content does not exist', () => {
        const type = registry_1.mockRegistry.types.dwaynejohnson;
        const tree = new treeContainers_1.VirtualTreeContainer([
            {
                dirPath: registry_1.DWAYNE_DIR,
                children: [registry_1.DWAYNE_XML_NAME],
            },
        ]);
        const adapter = new mixedContentSourceAdapter_1.MixedContentSourceAdapter(type, registry_1.mockRegistry, undefined, tree);
        chai_1.assert.throws(() => adapter.getComponent(registry_1.DWAYNE_SOURCE), errors_1.ExpectedSourceFilesError);
    });
    describe('File Content', () => {
        const type = registry_1.mockRegistry.types.dwaynejohnson;
        const tree = new treeContainers_1.VirtualTreeContainer([
            {
                dirPath: registry_1.DWAYNE_DIR,
                children: [registry_1.DWAYNE_XML_NAME, registry_1.DWAYNE_SOURCE_NAME],
            },
        ]);
        const adapter = new mixedContentSourceAdapter_1.MixedContentSourceAdapter(type, registry_1.mockRegistry, undefined, tree);
        const expectedComponent = new metadata_registry_1.SourceComponent({
            name: 'a',
            type,
            xml: registry_1.DWAYNE_XML,
            content: registry_1.DWAYNE_SOURCE,
        }, tree);
        it('Should return expected SourceComponent when given a root metadata xml path', () => {
            chai_1.expect(adapter.getComponent(registry_1.DWAYNE_XML)).to.deep.equal(expectedComponent);
        });
        it('Should return expected SourceComponent when given a source path', () => {
            chai_1.expect(adapter.getComponent(registry_1.DWAYNE_SOURCE)).to.deep.equal(expectedComponent);
        });
    });
    describe('Directory Content', () => {
        const { TARAJI_COMPONENT, TARAJI_SOURCE_PATHS, TARAJI_XML_PATHS, TARAJI_VIRTUAL_FS } = registry_1.taraji;
        const type = registry_1.mockRegistry.types.tarajihenson;
        const tree = new treeContainers_1.VirtualTreeContainer(TARAJI_VIRTUAL_FS);
        const adapter = new mixedContentSourceAdapter_1.MixedContentSourceAdapter(type, registry_1.mockRegistry, undefined, tree);
        const expectedComponent = new metadata_registry_1.SourceComponent(TARAJI_COMPONENT, tree);
        it('Should return expected SourceComponent when given a root metadata xml path', () => {
            chai_1.expect(adapter.getComponent(TARAJI_XML_PATHS[0])).to.deep.equal(expectedComponent);
        });
        it('Should return expected SourceComponent when given a source path', () => {
            const randomSource = TARAJI_SOURCE_PATHS[Math.floor(Math.random() * Math.floor(TARAJI_SOURCE_PATHS.length))];
            chai_1.expect(adapter.getComponent(randomSource)).to.deep.equal(expectedComponent);
        });
        it('should return expected SourceComponent when there is no metadata xml', () => {
            const tree = new treeContainers_1.VirtualTreeContainer(tarajiConstants_1.TARAJI_VIRTUAL_FS_NO_XML);
            const adapter = new mixedContentSourceAdapter_1.MixedContentSourceAdapter(type, registry_1.mockRegistry, undefined, tree);
            const expectedComponent = new metadata_registry_1.SourceComponent({
                name: 'a',
                type,
                content: tarajiConstants_1.TARAJI_CONTENT_PATH,
            }, tree);
            chai_1.expect(adapter.getComponent(registry_1.taraji.TARAJI_CONTENT_PATH)).to.deep.equal(expectedComponent);
        });
    });
});
//# sourceMappingURL=mixedContentSourceAdapter.js.map