"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const forceIgnore_1 = require("../../src/metadata-registry/forceIgnore");
const chai_1 = require("chai");
const sinon_1 = require("sinon");
const fs = require("fs");
const fsUtil = require("../../src/utils/fileSystemHandler");
const constants_1 = require("../../src/utils/constants");
const path_1 = require("path");
const core_1 = require("@salesforce/core");
const env = sinon_1.createSandbox();
describe('ForceIgnore', () => {
    const forceIgnorePath = path_1.join('some', constants_1.FORCE_IGNORE_FILE);
    const testPath = path_1.join('some', 'path', '__tests__', 'myTest.x');
    const testPattern = '**/__tests__/**';
    afterEach(() => env.restore());
    it('Should default to not ignoring a file if forceignore is not loaded', () => {
        const path = path_1.join('some', 'path');
        const forceIgnore = new forceIgnore_1.ForceIgnore();
        chai_1.expect(forceIgnore.accepts(path)).to.be.true;
        chai_1.expect(forceIgnore.denies(path)).to.be.false;
    });
    it('Should ignore files with a given pattern', () => {
        const readStub = env.stub(fs, 'readFileSync');
        readStub.withArgs(forceIgnorePath).returns(testPattern);
        const forceIgnore = new forceIgnore_1.ForceIgnore(forceIgnorePath);
        chai_1.expect(forceIgnore.accepts(testPath)).to.be.false;
        chai_1.expect(forceIgnore.denies(testPath)).to.be.true;
    });
    it('Should find a forceignore file from a given path', () => {
        const readStub = env.stub(fs, 'readFileSync');
        const searchStub = env.stub(fsUtil, 'searchUp');
        readStub.withArgs(forceIgnorePath).returns(testPattern);
        searchStub.withArgs(testPath, constants_1.FORCE_IGNORE_FILE).returns(forceIgnorePath);
        const forceIgnore = forceIgnore_1.ForceIgnore.findAndCreate(testPath);
        chai_1.expect(forceIgnore.accepts(testPath)).to.be.false;
    });
    it('should send telemetry event when the old parser and the new parser have different results', () => {
        const readStub = env.stub(fs, 'readFileSync');
        readStub.withArgs(forceIgnorePath).returns(testPattern);
        const forceIgnore = new forceIgnore_1.ForceIgnore(forceIgnorePath);
        let expected = {
            eventName: 'FORCE_IGNORE_DIFFERENCE',
            content: '**/__tests__/**\n**/*.dup\n**/.*\n**/package2-descriptor.json\n**/package2-manifest.json',
            oldLibraryResults: false,
            newLibraryResults: true,
            ignoreLines: [
                '**/__tests__/**',
                '**/*.dup',
                '**/.*',
                '**/package2-descriptor.json',
                '**/package2-manifest.json',
            ],
            file: 'some/path/__tests__/myTest.x',
        };
        if (process.platform === 'win32') {
            expected = {
                content: '**\\__tests__\\**\n**\\*.dup\n**\\.*\n**\\package2-descriptor.json\n**\\package2-manifest.json',
                eventName: 'FORCE_IGNORE_DIFFERENCE',
                file: 'some\\path\\__tests__\\myTest.x',
                ignoreLines: [
                    '**\\__tests__\\**',
                    '**\\*.dup',
                    '**\\.*',
                    '**\\package2-descriptor.json',
                    '**\\package2-manifest.json',
                ],
                newLibraryResults: true,
                oldLibraryResults: false,
            };
        }
        const telemetrySpy = env.spy(core_1.Lifecycle.prototype, 'emit');
        // @ts-ignore call the private method directly to avoid excessive stubbing
        forceIgnore.resolveConflict(true, false, testPath);
        chai_1.expect(telemetrySpy.calledOnce).to.be.true;
        chai_1.expect(telemetrySpy.args[0][0]).to.equal('telemetry');
        chai_1.expect(telemetrySpy.args[0][1]).to.deep.equal(expected);
    });
    /**
     * TODO: Rework when approach to default patterns changes. We should be able
     * to generally test the defaults system.
     */
    describe('Defaults', () => {
        let forceIgnore;
        const root = path_1.join('some', 'path');
        beforeEach(() => {
            env.stub(fs, 'readFileSync').returns('');
            forceIgnore = new forceIgnore_1.ForceIgnore();
        });
        it('Should ignore files starting with a dot', () => {
            const dotPath = path_1.join(root, '.xyz');
            chai_1.expect(forceIgnore.accepts(dotPath)).to.be.false;
            chai_1.expect(forceIgnore.denies(dotPath)).to.be.true;
        });
        it('Should ignore files ending in .dup', () => {
            const dupPath = path_1.join(root, 'abc.dup');
            chai_1.expect(forceIgnore.accepts(dupPath)).to.be.false;
            chai_1.expect(forceIgnore.denies(dupPath)).to.be.true;
        });
        it('Should ignore files named package2-descriptor.json', () => {
            const descriptorPath = path_1.join(root, 'package2-descriptor.json');
            chai_1.expect(forceIgnore.accepts(descriptorPath)).to.be.false;
            chai_1.expect(forceIgnore.denies(descriptorPath)).to.be.true;
        });
        it('Should ignore files named package2-manifest.json', () => {
            const manifestPath = path_1.join(root, 'package2-manifest.json');
            chai_1.expect(forceIgnore.accepts(manifestPath)).to.be.false;
            chai_1.expect(forceIgnore.denies(manifestPath)).to.be.true;
        });
    });
});
//# sourceMappingURL=forceIgnore.test.js.map