"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const manifestGenerator_1 = require("../../src/metadata-registry/manifestGenerator");
const chai_1 = require("chai");
const metadata_registry_1 = require("../../src/metadata-registry");
const sinon_1 = require("sinon");
const fs = require("fs");
const path = require("path");
const assert_1 = require("assert");
describe('ManifestGenerator', () => {
    let sandboxStub;
    const manifestGenerator = new manifestGenerator_1.ManifestGenerator();
    const registryAccess = new metadata_registry_1.RegistryAccess();
    const apiVersion = registryAccess.getApiVersion();
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('should generate manifest for one type', () => {
        const component = {
            fullName: 'someName',
            type: { id: 'foobar', name: 'FooBar' },
        };
        let expectedManifest = '<?xml version="1.0" encoding="UTF-8"?>\n';
        expectedManifest += '<Package xmlns="http://soap.sforce.com/2006/04/metadata">\n';
        expectedManifest +=
            '  <types>\n    <members>someName</members>\n    <name>FooBar</name>\n  </types>\n';
        expectedManifest += `  <version>${apiVersion}</version>\n</Package>`;
        chai_1.expect(manifestGenerator.createManifest([component])).to.equal(expectedManifest);
    });
    it('should generate manifest for multiple types', () => {
        const component1 = {
            fullName: 'apexClass1',
            type: { id: 'apexclass', name: 'ApexClass' },
        };
        const component2 = {
            fullName: 'apexTrigger1',
            type: { id: 'apextrigger', name: 'ApexTrigger' },
        };
        let expectedManifest = '<?xml version="1.0" encoding="UTF-8"?>\n';
        expectedManifest += '<Package xmlns="http://soap.sforce.com/2006/04/metadata">\n';
        expectedManifest +=
            '  <types>\n    <members>apexClass1</members>\n    <name>ApexClass</name>\n  </types>\n';
        expectedManifest +=
            '  <types>\n    <members>apexTrigger1</members>\n    <name>ApexTrigger</name>\n  </types>\n';
        expectedManifest += `  <version>${apiVersion}</version>\n</Package>`;
        chai_1.expect(manifestGenerator.createManifest([component1, component2])).to.equal(expectedManifest);
    });
    it('should generate manifest for multiple components', () => {
        const component1 = {
            fullName: 'apexClass1',
            type: { id: 'apexclass', name: 'ApexClass' },
        };
        const component2 = {
            fullName: 'apexClass2',
            type: { id: 'apexclass', name: 'ApexClass' },
        };
        const component3 = {
            fullName: 'apexTrigger1',
            type: { id: 'apextrigger', name: 'ApexTrigger' },
        };
        let expectedManifest = '<?xml version="1.0" encoding="UTF-8"?>\n';
        expectedManifest += '<Package xmlns="http://soap.sforce.com/2006/04/metadata">\n';
        expectedManifest +=
            '  <types>\n    <members>apexClass1</members>\n    <members>apexClass2</members>\n    <name>ApexClass</name>\n  </types>\n';
        expectedManifest +=
            '  <types>\n    <members>apexTrigger1</members>\n    <name>ApexTrigger</name>\n  </types>\n';
        expectedManifest += `  <version>${apiVersion}</version>\n</Package>`;
        chai_1.expect(manifestGenerator.createManifest([component1, component2, component3])).to.equal(expectedManifest);
    });
    it('should generate manifest for multiple components passed in different order', () => {
        const component1 = {
            fullName: 'apexClass1',
            type: { id: 'apexclass', name: 'ApexClass' },
        };
        const component2 = {
            fullName: 'apexClass2',
            type: { id: 'apexclass', name: 'ApexClass' },
        };
        const component3 = {
            fullName: 'apexTrigger1',
            type: { id: 'apextrigger', name: 'ApexTrigger' },
        };
        let expectedManifest = '<?xml version="1.0" encoding="UTF-8"?>\n';
        expectedManifest += '<Package xmlns="http://soap.sforce.com/2006/04/metadata">\n';
        expectedManifest +=
            '  <types>\n    <members>apexClass1</members>\n    <members>apexClass2</members>\n    <name>ApexClass</name>\n  </types>\n';
        expectedManifest +=
            '  <types>\n    <members>apexTrigger1</members>\n    <name>ApexTrigger</name>\n  </types>\n';
        expectedManifest += `  <version>${apiVersion}</version>\n</Package>`;
        chai_1.expect(manifestGenerator.createManifest([component1, component3, component2])).to.equal(expectedManifest);
    });
    it('should generate manifest by overriding apiversion', () => {
        const component = {
            fullName: 'someName',
            type: { id: 'apexclass', name: 'ApexClass' },
        };
        let expectedManifest = '<?xml version="1.0" encoding="UTF-8"?>\n';
        expectedManifest += '<Package xmlns="http://soap.sforce.com/2006/04/metadata">\n';
        expectedManifest +=
            '  <types>\n    <members>someName</members>\n    <name>ApexClass</name>\n  </types>\n';
        expectedManifest += '  <version>45.0</version>\n</Package>';
        chai_1.expect(manifestGenerator.createManifest([component], '45.0')).to.equal(expectedManifest);
    });
    const rootPath = path.join('file', 'path');
    const mdComponents = [
        metadata_registry_1.SourceComponent.createVirtualComponent({
            type: metadata_registry_1.registryData.types.apexclass,
            name: 'myTestClass',
            xml: path.join(rootPath, 'myTestClass.cls-meta.xml'),
            content: path.join(rootPath, 'myTestClass.cls'),
        }, [
            {
                dirPath: rootPath,
                children: ['myTestClass.cls', 'myTestClass.cls-meta.xml'],
            },
        ]),
    ];
    it('should successfully create a manifest with a sourcepath', () => {
        const registryAccess = new metadata_registry_1.RegistryAccess();
        sandboxStub.stub(registryAccess, 'getComponentsFromPath').returns(mdComponents);
        const manifestGenerator = new manifestGenerator_1.ManifestGenerator(registryAccess);
        const writeFileStub = sandboxStub.stub(fs, 'writeFileSync');
        manifestGenerator.createManifestFromPath(path.join('file', 'path', 'myTestClass.cls-meta.xml'), path.join(rootPath, 'manifest', 'package.xml'));
        chai_1.expect(writeFileStub.callCount).to.equal(1);
        chai_1.expect(writeFileStub.getCall(0).args[0]).to.equal(path.join(rootPath, 'manifest', 'package.xml'));
    });
    it('should throw error when handling unexpected errors on creating a manifest with a sourcepath', () => {
        const registryAccess = new metadata_registry_1.RegistryAccess();
        sandboxStub.stub(registryAccess, 'getComponentsFromPath').returns(mdComponents);
        const manifestGenerator = new manifestGenerator_1.ManifestGenerator(registryAccess);
        const writeFileStub = sandboxStub.stub(fs, 'writeFileSync');
        writeFileStub.onCall(0).throwsException('Unexpected error when creating file');
        const filePath = path.join(rootPath, 'myTestClass.cls-meta.xml');
        try {
            manifestGenerator.createManifestFromPath(filePath, path.join(rootPath, 'manifest', 'package.xml'));
            assert_1.fail('Test should have thrown an error before this line');
        }
        catch (e) {
            chai_1.expect(e.message).to.contain(`Unexpected error while creating manifest for '${filePath}'. Stack trace: Unexpected error when creating file`);
        }
    });
});
//# sourceMappingURL=manifestGenerator.test.js.map