"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const treeContainers_1 = require("../../src/metadata-registry/treeContainers");
const chai_1 = require("chai");
const sinon_1 = require("sinon");
const fs = require("fs");
const path_1 = require("path");
const errors_1 = require("../../src/errors");
const i18n_1 = require("../../src/i18n");
const stream_1 = require("stream");
const unzipper = require("unzipper");
const archiver_1 = require("archiver");
const util_1 = require("util");
describe('Tree Containers', () => {
    const readDirResults = ['a.q', 'a.x-meta.xml', 'b', 'b.x-meta.xml', 'c.z', 'c.x-meta.xml'];
    describe('BaseTreeContainer', () => {
        class TestTreeContainer extends treeContainers_1.BaseTreeContainer {
            readDirectory() {
                return readDirResults;
            }
            exists() {
                return false;
            }
            isDirectory() {
                return false;
            }
            readFile() {
                return Promise.resolve(Buffer.from(''));
            }
            stream() {
                return;
            }
        }
        const tree = new TestTreeContainer();
        it('should find first matching metadata file', () => {
            chai_1.expect(tree.find('metadata', 'b', '')).to.equal('b.x-meta.xml');
        });
        it('should find first matching content file', () => {
            chai_1.expect(tree.find('content', 'c', '')).to.equal('c.z');
        });
    });
    describe('NodeFSTreeContainer', () => {
        const env = sinon_1.createSandbox();
        const tree = new treeContainers_1.NodeFSTreeContainer();
        const path = path_1.join('path', 'to', 'test');
        afterEach(() => env.restore());
        it('should use expected Node API for exists', () => {
            const existsStub = env.stub(fs, 'existsSync');
            existsStub.withArgs(path).returns(true);
            chai_1.expect(tree.exists(path)).to.be.true;
            chai_1.expect(existsStub.calledOnce).to.be.true;
        });
        it('should use expected Node API for isDirectory', () => {
            const statStub = env.stub(fs, 'lstatSync');
            // @ts-ignore lstat returns more than isDirectory function
            statStub.withArgs(path).returns({ isDirectory: () => true });
            chai_1.expect(tree.isDirectory(path)).to.be.true;
            chai_1.expect(statStub.calledOnce).to.be.true;
        });
        it('should use expected Node API for readDirectory', () => {
            const readdirStub = env.stub(fs, 'readdirSync');
            // @ts-ignore wants Dirents but string[] works as well
            readdirStub.withArgs(path).returns(readDirResults);
            chai_1.expect(tree.readDirectory(path)).to.deep.equal(readDirResults);
            chai_1.expect(readdirStub.calledOnce).to.be.true;
        });
        it('should use expected Node API for readFile', () => __awaiter(void 0, void 0, void 0, function* () {
            const readFileStub = env.stub(fs, 'readFileSync');
            // @ts-ignore wants Dirents but string[] works as well
            readFileStub.withArgs(path).resolves(Buffer.from('test'));
            const data = yield tree.readFile(path);
            chai_1.expect(data.toString()).to.deep.equal('test');
            chai_1.expect(readFileStub.calledOnce).to.be.true;
        }));
        it('should use expected Node API for stream', () => __awaiter(void 0, void 0, void 0, function* () {
            const readable = new stream_1.Readable();
            const createReadStreamStub = env.stub(fs, 'createReadStream');
            // @ts-ignore wants ReadStream but Readable works for testing
            createReadStreamStub.withArgs(path).returns(readable);
            chai_1.expect(tree.stream(path)).to.deep.equal(readable);
        }));
    });
    describe('ZipTreeContainer', () => {
        let tree;
        let zipBuffer;
        const filesRoot = path_1.join('.', 'main', 'default');
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            const pipeline = util_1.promisify(stream_1.pipeline);
            const archive = archiver_1.create('zip', { zlib: { level: 3 } });
            const bufferWritable = new stream_1.Writable();
            const buffers = [];
            bufferWritable._write = (chunk, encoding, cb) => {
                buffers.push(chunk);
                cb();
            };
            pipeline(archive, bufferWritable);
            archive.append(null, { name: 'main/' });
            archive.append(null, { name: 'main/default/' });
            archive.append('test text', { name: 'main/default/test.txt' });
            archive.append('test text 2', { name: 'main/default/test2.txt' });
            archive.append('test text 3', { name: 'main/default/morefiles/test3.txt' });
            yield archive.finalize();
            zipBuffer = Buffer.concat(buffers);
            tree = yield treeContainers_1.ZipTreeContainer.create(zipBuffer);
        }));
        describe('exists', () => {
            it('should return true for file that exists', () => {
                const path = path_1.join(filesRoot, 'test.txt');
                chai_1.expect(tree.exists(path)).to.be.true;
            });
            it('should return true for directory that exists', () => {
                const path = path_1.join(filesRoot, 'morefiles');
                chai_1.expect(tree.exists(path)).to.be.true;
            });
            it('should return false for file that does not exist', () => {
                const path = path_1.join(filesRoot, 'test4.txt');
                chai_1.expect(tree.exists(path)).to.be.false;
            });
            it('should return false for directory that does not exist', () => {
                const path = path_1.join('.', 'dne');
                chai_1.expect(tree.exists(path)).to.be.false;
            });
        });
        describe('isDirectory', () => {
            it('should return false for isDirectory', () => {
                const path = path_1.join(filesRoot, 'test.txt');
                chai_1.expect(tree.isDirectory(path)).to.be.false;
            });
            it('should return true for isDirectory', () => {
                chai_1.expect(tree.isDirectory(filesRoot)).to.be.true;
            });
            it('should throw an error if path does not exist', () => {
                const path = 'dne';
                chai_1.assert.throws(() => tree.isDirectory(path), errors_1.LibraryError, i18n_1.nls.localize('error_path_not_found', path));
            });
        });
        describe('readDirectory', () => {
            it('should return correct directory entries for directory with files and directories', () => {
                chai_1.expect(tree.readDirectory(filesRoot)).to.deep.equal(['test.txt', 'test2.txt', 'morefiles']);
            });
            it('should return correct directory entries for directory only files', () => {
                const path = path_1.join(filesRoot, 'morefiles');
                chai_1.expect(tree.readDirectory(path)).to.deep.equal(['test3.txt']);
            });
            it('should return correct directory entries for directory with only directories', () => {
                chai_1.expect(tree.readDirectory('.')).to.deep.equal(['main']);
            });
            it('should throw an error if path is not a directory', () => {
                const path = path_1.join(filesRoot, 'test2.txt');
                chai_1.assert.throws(() => tree.readDirectory(path), errors_1.LibraryError, i18n_1.nls.localize('error_expected_directory_path', path));
            });
        });
        describe('readFile', () => {
            it('should read contents of zip entry into buffer', () => __awaiter(void 0, void 0, void 0, function* () {
                const path = path_1.join(filesRoot, 'test.txt');
                const contents = (yield tree.readFile(path)).toString();
                chai_1.expect(contents).to.equal('test text');
            }));
            it('should throw an error if path is to directory', () => {
                chai_1.assert.throws(() => tree.readFile(filesRoot), errors_1.LibraryError, i18n_1.nls.localize('error_expected_file_path', filesRoot));
            });
        });
        describe('stream', () => {
            it('should return a readable stream', () => __awaiter(void 0, void 0, void 0, function* () {
                var _a;
                const path = path_1.join(filesRoot, 'test.txt');
                const zipDir = yield unzipper.Open.buffer(zipBuffer);
                const expectedStream = (_a = zipDir.files.find((f) => path_1.normalize(f.path) === path)) === null || _a === void 0 ? void 0 : _a.stream();
                const actual = tree.stream(path);
                chai_1.expect(actual instanceof stream_1.Readable).to.be.true;
                chai_1.expect(actual.path).to.equal(expectedStream.path);
            }));
            it('should throw an error if given path is to a directory', () => {
                chai_1.assert.throws(() => tree.stream(filesRoot), errors_1.LibraryError, i18n_1.nls.localize('error_no_directory_stream', tree.constructor.name));
            });
        });
    });
    describe('VirtualTreeContainer', () => {
        const virtualFS = [
            {
                dirPath: '.',
                children: ['test.txt', 'test2.txt', 'files', 'logs'],
            },
            {
                dirPath: path_1.join('.', 'files'),
                children: ['test3.txt'],
            },
            {
                dirPath: path_1.join('.', 'logs'),
                children: [{ name: 'run.log', data: Buffer.from('successful') }],
            },
        ];
        const tree = new treeContainers_1.VirtualTreeContainer(virtualFS);
        describe('exists', () => {
            it('should return true for file that exists', () => {
                const path = path_1.join('.', 'test.txt');
                chai_1.expect(tree.exists(path)).to.be.true;
            });
            it('should return true for directory that exists', () => {
                const path = path_1.join('.', 'files');
                chai_1.expect(tree.exists(path)).to.be.true;
            });
            it('should return false for file that does not exist', () => {
                const path = path_1.join('.', 'files', 'text4.txt');
                chai_1.expect(tree.exists(path)).to.be.false;
            });
            it('should return false for directory that does not exist', () => {
                const path = path_1.join('.', 'otherfiles');
                chai_1.expect(tree.exists(path)).to.be.false;
            });
        });
        describe('isDirectory', () => {
            it('should return false for isDirectory', () => {
                const path = path_1.join('.', 'test.txt');
                chai_1.expect(tree.isDirectory(path)).to.be.false;
            });
            it('should return true for isDirectory', () => {
                const path = path_1.join('.', 'files');
                chai_1.expect(tree.isDirectory(path)).to.be.true;
            });
            it('should throw an error if path does not exist', () => {
                const path = 'dne';
                chai_1.assert.throws(() => tree.isDirectory(path), errors_1.LibraryError, i18n_1.nls.localize('error_path_not_found', path));
            });
        });
        describe('readDirectory', () => {
            it('should return directory entries for readDirectory', () => {
                chai_1.expect(tree.readDirectory('.')).to.deep.equal(virtualFS[0].children);
            });
        });
        describe('readFile', () => {
            it('should return file data for given path', () => __awaiter(void 0, void 0, void 0, function* () {
                const data = yield tree.readFile(path_1.join('.', 'logs', 'run.log'));
                chai_1.expect(data.toString()).to.equal('successful');
            }));
            it('should return empty string buffer if file initialize without data', () => __awaiter(void 0, void 0, void 0, function* () {
                const data = yield tree.readFile(path_1.join('.', 'test.txt'));
                chai_1.expect(data.toString()).to.equal('');
            }));
            it('should throw error if path does not exist', () => __awaiter(void 0, void 0, void 0, function* () {
                const path = 'dne';
                try {
                    yield tree.readFile(path);
                    chai_1.assert.fail('should have thrown an error');
                }
                catch (e) {
                    chai_1.expect(e.message).to.deep.equal(i18n_1.nls.localize('error_path_not_found', path));
                }
            }));
        });
        describe('stream', () => {
            it('should throw a not implemented error', () => {
                chai_1.assert.throws(() => tree.stream('file'), 'Method not implemented');
            });
        });
    });
});
//# sourceMappingURL=treeContainers.test.js.map