"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const fsUtil = require("../../src/utils/fileSystemHandler");
const sinon_1 = require("sinon");
const chai_1 = require("chai");
const path_1 = require("path");
const fs = require("fs");
const env = sinon_1.createSandbox();
describe('File System Utils', () => {
    const root = path_1.join(process.cwd(), 'path', 'to', 'whatever');
    afterEach(() => env.restore());
    describe('isDirectory', () => {
        it('Should utilize fs Stats to determine if path is a directory', () => {
            const statStub = env.stub(fs, 'lstatSync');
            // @ts-ignore
            statStub.withArgs(root).returns({ isDirectory: () => true });
            chai_1.expect(fsUtil.isDirectory(root)).to.be.true;
        });
    });
    describe('searchUp', () => {
        let existsStub;
        const filePath = path_1.join(root, 'test.x');
        const startPath = path_1.join(root, 'a', 'more', 'nested', 'file.y');
        beforeEach(() => {
            existsStub = env.stub(fs, 'existsSync');
            existsStub.returns(false);
        });
        it('Should traverse up and find a file with the given file name', () => {
            existsStub.withArgs(filePath).returns(true);
            chai_1.expect(fsUtil.searchUp(startPath, 'test.x')).to.equal(filePath);
        });
        it('Should return start path if it is the file being searched for', () => {
            existsStub.withArgs(filePath).returns(true);
            chai_1.expect(fsUtil.searchUp(filePath, 'test.x')).to.equal(filePath);
        });
        it('Should return undefined if file not found', () => {
            chai_1.expect(fsUtil.searchUp(startPath, 'asdf')).to.be.undefined;
        });
    });
    describe('ensureDirectoryExists', () => {
        let mkdirStub;
        let existsStub;
        beforeEach(() => {
            mkdirStub = env.stub(fs, 'mkdirSync');
            existsStub = env.stub(fs, 'existsSync');
        });
        it('should return immediately if file or directory already exists', () => {
            const path = path_1.join('path', 'to', 'dir');
            existsStub.withArgs(path).returns(true);
            fsUtil.ensureDirectoryExists(path);
            chai_1.expect(mkdirStub.notCalled).to.be.true;
        });
        it('should create nested directories as needed', () => {
            const path = path_1.join('path', 'to');
            const path2 = path_1.join(path, 'dir');
            const path3 = path_1.join(path2, 'dir2');
            existsStub.returns(false);
            existsStub.withArgs(path).returns(true);
            fsUtil.ensureDirectoryExists(path3);
            chai_1.expect(mkdirStub.firstCall.args[0]).to.equal(path2);
            chai_1.expect(mkdirStub.secondCall.args[0]).to.equal(path3);
        });
    });
    describe('ensureFileExists', () => {
        it('should ensure file exists', () => {
            const path = path_1.join('path', 'to', 'a', 'file.x');
            const closeStub = env.stub(fs, 'closeSync');
            const openStub = env.stub(fs, 'openSync');
            openStub.returns(123);
            const existsSyncStub = env.stub(fs, 'existsSync').returns(true);
            fsUtil.ensureFileExists(path);
            // somewhat validating ensureDirectoryExists was called first
            chai_1.expect(existsSyncStub.calledBefore(openStub)).to.be.true;
            chai_1.expect(closeStub.firstCall.args[0]).to.equal(123);
        });
    });
    describe('deleteDirectory', () => {
        const path = path_1.join('path', 'to');
        const path2 = path_1.join(path, 'dir');
        let readdirStub;
        let existsStub;
        let rmdirStub;
        let unlinkStub;
        beforeEach(() => {
            // @ts-ignore
            readdirStub = env.stub(fs, 'readdirSync');
            existsStub = env.stub(fs, 'existsSync');
            rmdirStub = env.stub(fs, 'rmdirSync');
            unlinkStub = env.stub(fs, 'unlinkSync');
        });
        it('should return immediately if file or directory does not exist', () => {
            existsStub.withArgs(path2).returns(false);
            fsUtil.deleteDirectory(path2);
            chai_1.expect(readdirStub.notCalled).to.be.true;
            chai_1.expect(rmdirStub.notCalled).to.be.true;
        });
        it('should remove nested files as needed', () => {
            const testFile = 'fileOne';
            existsStub.returns(true);
            readdirStub.returns([testFile]);
            const statStub = env.stub(fs, 'lstatSync');
            // @ts-ignore
            statStub.returns({ isDirectory: () => false });
            fsUtil.deleteDirectory(path2);
            chai_1.expect(readdirStub.firstCall.args[0]).to.equal(path2);
            chai_1.expect(unlinkStub.firstCall.args[0]).to.equal(path_1.join(path2, testFile));
            chai_1.expect(rmdirStub.firstCall.args[0]).to.equal(path2);
        });
        it('should remove nested directories as needed', () => {
            const testDir = 'dirOne';
            const testDirPath = path_1.join(path2, testDir);
            existsStub.returns(true);
            readdirStub.withArgs(path2).returns([testDir]);
            readdirStub.withArgs(testDirPath).returns([]);
            env
                .stub(fs, 'lstatSync')
                .withArgs(path_1.join(path2, testDir))
                // @ts-ignore
                .returns({ isDirectory: () => true });
            fsUtil.deleteDirectory(path2);
            chai_1.expect(readdirStub.firstCall.args[0]).to.equal(path2);
            chai_1.expect(readdirStub.secondCall.args[0]).to.equal(testDirPath);
            chai_1.expect(rmdirStub.firstCall.args[0]).to.equal(testDirPath);
            chai_1.expect(rmdirStub.secondCall.args[0]).to.equal(path2);
            chai_1.expect(unlinkStub.notCalled).to.be.true;
        });
    });
});
//# sourceMappingURL=fileSystemHandler.js.map